@@ -2,7 +2,7 @@
  * vlc_text_style.h: text_style_t definition and helpers.
  *****************************************************************************
  * Copyright (C) 1999-2010 VLC authors and VideoLAN
- * $Id: 523d789147687553ef6da90b1297ea60ef47f1dd $
+ * $Id: b3b07bf8cd0252c4aafe66ccbcac92eec725672a $
  *
  * Authors: Derk-Jan Hartman <hartman _AT_ videolan _DOT_ org>
  *          basOS G <noxelia 4t gmail , com>
@@ -68,6 +68,14 @@ typedef struct
     uint8_t    i_background_alpha;/**< The transparency of the background */
     int        i_karaoke_background_color;/**< Background color for karaoke 0xRRGGBB */
     uint8_t    i_karaoke_background_alpha;/**< The transparency of the karaoke bg */
+
+    /* Line breaking */
+    enum
+    {
+        STYLE_WRAP_DEFAULT = 0,   /**< Breaks on whitespace or fallback on char */
+        STYLE_WRAP_CHAR,          /**< Breaks at character level only */
+        STYLE_WRAP_NONE,          /**< No line breaks (except explicit ones) */
+    } e_wrapinfo;
 } text_style_t;
 
 #define STYLE_ALPHA_OPAQUE      0xFF
@@ -87,6 +95,7 @@ typedef struct
 #define STYLE_HAS_BACKGROUND_ALPHA      (1 << 8)
 #define STYLE_HAS_K_BACKGROUND_COLOR    (1 << 9)
 #define STYLE_HAS_K_BACKGROUND_ALPHA    (1 << 10)
+#define STYLE_HAS_WRAP_INFO             (1 << 11)
 
 /* Style flags for \ref text_style_t */
 #define STYLE_BOLD              (1 << 0)
