@@ -2,7 +2,7 @@
  * imem.c : Memory input for VLC
  *****************************************************************************
  * Copyright (C) 2009-2010 Laurent Aimar
- * $Id: 9999c46cb6e69722555766f6b704e4eca804410b $
+ * $Id: 60ff35287e2486d7ccc12712e54bd45529139ffb $
  *
  * Author: Laurent Aimar <fenrir _AT_ videolan _DOT org>
  *
@@ -230,7 +230,6 @@ static void ParseMRL(vlc_object_t *, const char *);
 static void CloseCommon(imem_sys_t *sys)
 {
     free(sys->source.cookie);
-    free(sys);
 }
 
 /**
@@ -241,7 +240,7 @@ static int OpenCommon(vlc_object_t *object, imem_sys_t **sys_ptr, const char *ps
     char *tmp;
 
     /* */
-    imem_sys_t *sys = calloc(1, sizeof(*sys));
+    imem_sys_t *sys = vlc_calloc(object, 1, sizeof(*sys));
     if (!sys)
         return VLC_ENOMEM;
 
