@@ -4,7 +4,7 @@
  * Copyright (C) 2009 Geoffroy Couprie
  * Copyright (C) 2009 Laurent Aimar
  * Copyright (C) 2015 Steve Lhomme
- * $Id: d114feac79cd47574862e5b07c2ec61ff75fa412 $
+ * $Id: b8f7cc51d68d201e8a5d4d82d206ac93ecb8a63c $
  *
  * Authors: Geoffroy Couprie <geal@videolan.org>
  *          Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
@@ -36,6 +36,13 @@
 
 #define COBJMACROS
 
+#define D3D_Device          IUnknown
+#define D3D_DecoderType     IUnknown
+#define D3D_DecoderDevice   IUnknown
+#define D3D_DecoderSurface  IUnknown
+struct picture_sys_t {
+    void *dummy;
+};
 #include "directx_va.h"
 
 #include "avcodec.h"
@@ -52,6 +59,12 @@ static const int PROF_HEVC_MAIN[]    = { FF_PROFILE_HEVC_MAIN, 0 };
 static const int PROF_HEVC_MAIN10[]  = { FF_PROFILE_HEVC_MAIN,
                                          FF_PROFILE_HEVC_MAIN_10, 0 };
 
+#include <winapifamily.h>
+#if defined(WINAPI_FAMILY)
+# undef WINAPI_FAMILY
+#endif
+#define WINAPI_FAMILY WINAPI_PARTITION_DESKTOP
+
 #include <d3d9.h>
 #include <dxva2api.h>
 
@@ -148,7 +161,7 @@ DEFINE_GUID(DXVA_ModeVP9_VLD_Profile0,              0x463707f8, 0xa1d0, 0x4585,
 typedef struct {
     const char   *name;
     const GUID   *guid;
-    int          codec;
+    enum AVCodecID codec;
     const int    *p_profiles; // NULL or ends with 0
 } directx_va_mode_t;
 
@@ -250,11 +263,7 @@ static const directx_va_mode_t DXVA_MODES[] = {
     { NULL, NULL, 0, NULL }
 };
 
-static int FindVideoServiceConversion(vlc_va_t *, directx_sys_t *, const es_format_t *fmt);
-static void DestroyVideoDecoder(vlc_va_t *, directx_sys_t *);
-static void DestroyVideoService(vlc_va_t *, directx_sys_t *);
-static void DestroyDeviceManager(vlc_va_t *, directx_sys_t *);
-static void DestroyDevice(vlc_va_t *, directx_sys_t *);
+static int FindVideoServiceConversion(vlc_va_t *, directx_sys_t *, const es_format_t *, const AVCodecContext *);
 
 char *directx_va_GetDecoderName(const GUID *guid)
 {
@@ -270,26 +279,19 @@ char *directx_va_GetDecoderName(const GUID *guid)
 }
 
 /* */
-int directx_va_Setup(vlc_va_t *va, directx_sys_t *dx_sys, AVCodecContext *avctx)
+int directx_va_Setup(vlc_va_t *va, directx_sys_t *dx_sys, const AVCodecContext *avctx,
+                     const es_format_t *fmt)
 {
-    int surface_alignment = 16;
-    int surface_count = 2;
-
-    if (dx_sys->width == avctx->coded_width && dx_sys->height == avctx->coded_height
-     && dx_sys->decoder != NULL)
-        goto ok;
-
     /* */
-    DestroyVideoDecoder(va, dx_sys);
-
-    avctx->hwaccel_context = NULL;
-    if (avctx->coded_width <= 0 || avctx->coded_height <= 0)
+    if (FindVideoServiceConversion(va, dx_sys, fmt, avctx)) {
+        msg_Err(va, "FindVideoServiceConversion failed");
         return VLC_EGENERIC;
+    }
 
-    /* */
-    msg_Dbg(va, "directx_va_Setup id %d %dx%d", dx_sys->codec_id, avctx->coded_width, avctx->coded_height);
+    int surface_alignment = 16;
+    unsigned surface_count = 2;
 
-    switch ( dx_sys->codec_id )
+    switch ( avctx->codec_id )
     {
     case AV_CODEC_ID_MPEG2VIDEO:
         /* decoding MPEG-2 requires additional alignment on some Intel GPUs,
@@ -311,144 +313,21 @@ int directx_va_Setup(vlc_va_t *va, directx_sys_t *dx_sys, AVCodecContext *avctx)
     }
 
     if ( avctx->active_thread_type & FF_THREAD_FRAME )
-        surface_count += dx_sys->thread_count;
+        surface_count += avctx->thread_count;
 
-    if (surface_count > MAX_SURFACE_COUNT)
-        return VLC_EGENERIC;
-
-    dx_sys->surface_count = surface_count;
-
-#define ALIGN(x, y) (((x) + ((y) - 1)) & ~((y) - 1))
-    dx_sys->width  = avctx->coded_width;
-    dx_sys->height = avctx->coded_height;
-    dx_sys->surface_width  = ALIGN(dx_sys->width, surface_alignment);
-    dx_sys->surface_height = ALIGN(dx_sys->height, surface_alignment);
-
-    /* FIXME transmit a video_format_t by VaSetup directly */
-    video_format_t fmt;
-    memset(&fmt, 0, sizeof(fmt));
-    fmt.i_width = dx_sys->width;
-    fmt.i_height = dx_sys->height;
-    fmt.i_frame_rate = avctx->framerate.num;
-    fmt.i_frame_rate_base = avctx->framerate.den;
-
-    if (dx_sys->pf_create_decoder_surfaces(va, dx_sys->codec_id, &fmt))
-        return VLC_EGENERIC;
-
-    if (avctx->coded_width != dx_sys->surface_width ||
-        avctx->coded_height != dx_sys->surface_height)
-        msg_Warn( va, "surface dimensions (%dx%d) differ from avcodec dimensions (%dx%d)",
-                  dx_sys->surface_width, dx_sys->surface_height,
-                  avctx->coded_width, avctx->coded_height);
-
-    dx_sys->pf_setup_avcodec_ctx(va);
-
-    for (int i = 0; i < dx_sys->surface_count; i++) {
-        vlc_va_surface_t *surface = &dx_sys->surface[i];
-        surface->refcount = 0;
-        surface->order = 0;
-        surface->p_lock = &dx_sys->surface_lock;
-        surface->p_pic = dx_sys->pf_alloc_surface_pic(va, &fmt, i);
-        if (unlikely(surface->p_pic == NULL))
-            return VLC_EGENERIC;
-    }
-
-ok:
-    return VLC_SUCCESS;
-}
-
-void DestroyVideoDecoder(vlc_va_t *va, directx_sys_t *dx_sys)
-{
-    dx_sys->pf_destroy_surfaces(va);
-
-    for (int i = 0; i < dx_sys->surface_count; i++)
-        IUnknown_Release( dx_sys->hw_surface[i] );
-
-    for (int i = 0; i < dx_sys->surface_count; i++)
-        if (dx_sys->surface[i].p_pic)
-            picture_Release(dx_sys->surface[i].p_pic);
-
-    if (dx_sys->decoder)
-        IUnknown_Release( dx_sys->decoder );
-
-    dx_sys->decoder = NULL;
-    dx_sys->surface_count = 0;
-}
-
-/* FIXME it is nearly common with VAAPI */
-int directx_va_Get(vlc_va_t *va, directx_sys_t *dx_sys, picture_t *pic, uint8_t **data)
-{
-    /* Check the device */
-    if (dx_sys->pf_check_device(va)!=VLC_SUCCESS)
-        return VLC_EGENERIC;
-
-    vlc_mutex_lock( &dx_sys->surface_lock );
-
-    /* Grab the oldest unused surface, in case none are, use the oldest used one
-     * XXX using the used one is a workaround in case a problem happens with libavcodec */
-    int i, old = -1, old_used = -1;
-
-    for (i = 0; i < dx_sys->surface_count; i++) {
-        vlc_va_surface_t *surface = &dx_sys->surface[i];
-        if (((old == -1 || surface->order < dx_sys->surface[old].order)) && !surface->refcount)
-            old = i;
-        if (old_used == -1 || surface->order < dx_sys->surface[old_used].order)
-            old_used = i;
-    }
-    if (old >= 0)
-        i = old;
-    else if (old_used >= 0)
-    {
-        msg_Warn(va, "couldn't find a free decoding buffer, using index %d", old_used);
-        i = old_used;
-    }
-
-    vlc_va_surface_t *surface = &dx_sys->surface[i];
-
-    surface->refcount = 1;
-    surface->order = ++dx_sys->surface_order;
-    *data = (void *)dx_sys->hw_surface[i];
-    pic->context = surface;
-
-    vlc_mutex_unlock( &dx_sys->surface_lock );
-
-    return VLC_SUCCESS;
-}
-
-void directx_va_Release(void *opaque, uint8_t *data)
-{
-    picture_t *pic = opaque;
-    vlc_va_surface_t *surface = pic->context;
-    vlc_mutex_lock( surface->p_lock );
-
-    surface->refcount--;
-    pic->context = NULL;
-    picture_Release(pic);
-
-    vlc_mutex_unlock( surface->p_lock );
+    return va_pool_Setup(va, &dx_sys->va_pool, avctx,
+                         surface_count, surface_alignment);
 }
 
 void directx_va_Close(vlc_va_t *va, directx_sys_t *dx_sys)
 {
-    DestroyVideoDecoder(va, dx_sys);
-    DestroyVideoService(va, dx_sys);
-    DestroyDeviceManager(va, dx_sys);
-    DestroyDevice(va, dx_sys);
-
+    va_pool_Close(va, &dx_sys->va_pool);
     if (dx_sys->hdecoder_dll)
         FreeLibrary(dx_sys->hdecoder_dll);
-
-    vlc_mutex_destroy( &dx_sys->surface_lock );
 }
 
-int directx_va_Open(vlc_va_t *va, directx_sys_t *dx_sys,
-                    AVCodecContext *ctx, const es_format_t *fmt, bool b_dll)
+int directx_va_Open(vlc_va_t *va, directx_sys_t *dx_sys, bool b_dll)
 {
-    // TODO va->sys = sys;
-    dx_sys->codec_id = ctx->codec_id;
-
-    vlc_mutex_init( &dx_sys->surface_lock );
-
     if (b_dll) {
         /* Load dll*/
         dx_sys->hdecoder_dll = LoadLibrary(dx_sys->psz_decoder_dll);
@@ -459,30 +338,8 @@ int directx_va_Open(vlc_va_t *va, directx_sys_t *dx_sys,
         msg_Dbg(va, "DLLs loaded");
     }
 
-    /* */
-    if (dx_sys->pf_create_device(va)) {
-        msg_Err(va, "Failed to create DirectX device");
+    if (va_pool_Open(va, &dx_sys->va_pool) != VLC_SUCCESS)
         goto error;
-    }
-    msg_Dbg(va, "CreateDevice succeed");
-
-    if (dx_sys->pf_create_device_manager(va)) {
-        msg_Err(va, "D3dCreateDeviceManager failed");
-        goto error;
-    }
-
-    if (dx_sys->pf_create_video_service(va)) {
-        msg_Err(va, "DxCreateVideoService failed");
-        goto error;
-    }
-
-    /* */
-    if (FindVideoServiceConversion(va, dx_sys, fmt)) {
-        msg_Err(va, "FindVideoServiceConversion failed");
-        goto error;
-    }
-
-    dx_sys->thread_count = ctx->thread_count;
 
     return VLC_SUCCESS;
 
@@ -523,17 +380,11 @@ static bool profile_supported(const directx_va_mode_t *mode, const es_format_t *
     return is_supported;
 }
 
-void DestroyVideoService(vlc_va_t *va, directx_sys_t *dx_sys)
-{
-    dx_sys->pf_destroy_video_service(va);
-    if (dx_sys->d3ddec)
-        IUnknown_Release(dx_sys->d3ddec);
-}
-
 /**
  * Find the best suited decoder mode GUID and render format.
  */
-static int FindVideoServiceConversion(vlc_va_t *va, directx_sys_t *dx_sys, const es_format_t *fmt)
+static int FindVideoServiceConversion(vlc_va_t *va, directx_sys_t *dx_sys,
+                                      const es_format_t *fmt, const AVCodecContext *avctx)
 {
     input_list_t p_list = { 0 };
     int err = dx_sys->pf_get_input_list(va, &p_list);
@@ -556,7 +407,7 @@ static int FindVideoServiceConversion(vlc_va_t *va, directx_sys_t *dx_sys, const
     /* Try all supported mode by our priority */
     const directx_va_mode_t *mode = DXVA_MODES;
     for (; mode->name; ++mode) {
-        if (!mode->codec || mode->codec != dx_sys->codec_id)
+        if (!mode->codec || mode->codec != avctx->codec_id)
             continue;
 
         /* */
@@ -587,15 +438,3 @@ static int FindVideoServiceConversion(vlc_va_t *va, directx_sys_t *dx_sys, const
     p_list.pf_release(&p_list);
     return err;
 }
-
-void DestroyDeviceManager(vlc_va_t *va, directx_sys_t *dx_sys)
-{
-    dx_sys->pf_destroy_device_manager(va);
-}
-
-void DestroyDevice(vlc_va_t *va, directx_sys_t *dx_sys)
-{
-    dx_sys->pf_destroy_device(va);
-    if (dx_sys->d3ddev)
-        IUnknown_Release( dx_sys->d3ddev );
-}
