@@ -2,7 +2,7 @@
  * video.c: video decoder using the libavcodec library
  *****************************************************************************
  * Copyright (C) 1999-2001 VLC authors and VideoLAN
- * $Id: 52736c119bc4331af8f08c51318beb1c70447c16 $
+ * $Id: ef9a8a769a24348f3fe20b44c84e8e57ae5469b2 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -53,10 +53,11 @@
  *****************************************************************************/
 struct decoder_sys_t
 {
-    AVCODEC_COMMON_MEMBERS
+    AVCodecContext *p_context;
+    const AVCodec  *p_codec;
 
     /* Video decoder specific part */
-    mtime_t i_pts;
+    date_t  pts;
 
     /* Closed captions for decoders */
     cc_data_t cc;
@@ -305,6 +306,16 @@ static int lavc_UpdateVideoFormat(decoder_t *dec, AVCodecContext *ctx,
     if (val)
         return val;
 
+    /* always have date in fields/ticks units */
+    if(dec->p_sys->pts.i_divider_num)
+        date_Change(&dec->p_sys->pts, fmt_out.i_frame_rate *
+                                      __MAX(ctx->ticks_per_frame, 1),
+                                      fmt_out.i_frame_rate_base);
+    else
+        date_Init(&dec->p_sys->pts, fmt_out.i_frame_rate *
+                                    __MAX(ctx->ticks_per_frame, 1),
+                                    fmt_out.i_frame_rate_base);
+
     const int i_cc_reorder = dec->fmt_out.subs.cc.i_reorder_depth;
     fmt_out.p_palette = dec->fmt_out.video.p_palette;
     dec->fmt_out.video.p_palette = NULL;
@@ -362,54 +373,56 @@ static int lavc_CopyPicture(decoder_t *dec, picture_t *pic, AVFrame *frame)
 static int OpenVideoCodec( decoder_t *p_dec )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
+    AVCodecContext *ctx = p_sys->p_context;
+    const AVCodec *codec = p_sys->p_codec;
     int ret;
 
-    if( p_sys->p_context->extradata_size <= 0 )
+    if( ctx->extradata_size <= 0 )
     {
-        if( p_sys->p_codec->id == AV_CODEC_ID_VC1 ||
-            p_sys->p_codec->id == AV_CODEC_ID_THEORA )
+        if( codec->id == AV_CODEC_ID_VC1 ||
+            codec->id == AV_CODEC_ID_THEORA )
         {
             msg_Warn( p_dec, "waiting for extra data for codec %s",
-                      p_sys->p_codec->name );
+                      codec->name );
             return 1;
         }
     }
 
-    p_sys->p_context->width  = p_dec->fmt_in.video.i_visible_width;
-    p_sys->p_context->height = p_dec->fmt_in.video.i_visible_height;
+    ctx->width  = p_dec->fmt_in.video.i_visible_width;
+    ctx->height = p_dec->fmt_in.video.i_visible_height;
 
-    p_sys->p_context->coded_width = p_dec->fmt_in.video.i_width;
-    p_sys->p_context->coded_height = p_dec->fmt_in.video.i_height;
+    ctx->coded_width = p_dec->fmt_in.video.i_width;
+    ctx->coded_height = p_dec->fmt_in.video.i_height;
 
-    p_sys->p_context->bits_per_coded_sample = p_dec->fmt_in.video.i_bits_per_pixel;
+    ctx->bits_per_coded_sample = p_dec->fmt_in.video.i_bits_per_pixel;
     p_sys->pix_fmt = AV_PIX_FMT_NONE;
     p_sys->profile = -1;
     p_sys->level = -1;
     cc_Init( &p_sys->cc );
 
     post_mt( p_sys );
-    ret = ffmpeg_OpenCodec( p_dec );
+    ret = ffmpeg_OpenCodec( p_dec, ctx, codec );
     wait_mt( p_sys );
     if( ret < 0 )
         return ret;
 
-    switch( p_sys->p_context->active_thread_type )
+    switch( ctx->active_thread_type )
     {
         case FF_THREAD_FRAME:
             msg_Dbg( p_dec, "using frame thread mode with %d threads",
-                     p_sys->p_context->thread_count );
+                     ctx->thread_count );
             break;
         case FF_THREAD_SLICE:
             msg_Dbg( p_dec, "using slice thread mode with %d threads",
-                     p_sys->p_context->thread_count );
+                     ctx->thread_count );
             break;
         case 0:
-            if( p_sys->p_context->thread_count > 1 )
+            if( ctx->thread_count > 1 )
                 msg_Warn( p_dec, "failed to enable threaded decoding" );
             break;
         default:
             msg_Warn( p_dec, "using unknown thread mode with %d threads",
-                      p_sys->p_context->thread_count );
+                      ctx->thread_count );
             break;
     }
     return 0;
@@ -421,19 +434,27 @@ static int OpenVideoCodec( decoder_t *p_dec )
  * the ffmpeg codec will be opened, some memory allocated. The vout is not yet
  * opened (done after the first decoded frame).
  *****************************************************************************/
-int InitVideoDec( decoder_t *p_dec, AVCodecContext *p_context,
-                  const AVCodec *p_codec )
+int InitVideoDec( vlc_object_t *obj )
 {
-    decoder_sys_t *p_sys;
+    decoder_t *p_dec = (decoder_t *)obj;
+    const AVCodec *p_codec;
+    AVCodecContext *p_context = ffmpeg_AllocContext( p_dec, &p_codec );
+    if( p_context == NULL )
+        return VLC_EGENERIC;
+
     int i_val;
 
     /* Allocate the memory needed to store the decoder's structure */
-    if( ( p_dec->p_sys = p_sys = calloc( 1, sizeof(decoder_sys_t) ) ) == NULL )
+    decoder_sys_t *p_sys = calloc( 1, sizeof(*p_sys) );
+    if( unlikely(p_sys == NULL) )
+    {
+        avcodec_free_context( &p_context );
         return VLC_ENOMEM;
+    }
 
+    p_dec->p_sys = p_sys;
     p_sys->p_context = p_context;
     p_sys->p_codec = p_codec;
-    p_sys->b_delayed_open = true;
     p_sys->p_va = NULL;
     vlc_sem_init( &p_sys->sem_mt, 0 );
 
@@ -545,7 +566,8 @@ int InitVideoDec( decoder_t *p_dec, AVCodecContext *p_context,
         p_dec->i_extra_picture_buffers = 2 * p_context->thread_count;
 
     /* ***** misc init ***** */
-    p_sys->i_pts = VLC_TS_INVALID;
+    date_Init(&p_sys->pts, 1, 30001);
+    date_Set(&p_sys->pts, VLC_TS_INVALID);
     p_sys->b_first_frame = true;
     p_sys->i_late_frames = 0;
     p_sys->b_from_preroll = false;
@@ -577,12 +599,18 @@ int InitVideoDec( decoder_t *p_dec, AVCodecContext *p_context,
     {
         vlc_sem_destroy( &p_sys->sem_mt );
         free( p_sys );
+        avcodec_free_context( &p_context );
         return VLC_EGENERIC;
     }
 
     p_dec->pf_decode = DecodeVideo;
     p_dec->pf_flush  = Flush;
 
+    /* XXX: Writing input format makes little sense. */
+    if( p_context->profile != FF_PROFILE_UNKNOWN )
+        p_dec->fmt_in.i_profile = p_context->profile;
+    if( p_context->level != FF_LEVEL_UNKNOWN )
+        p_dec->fmt_in.i_level = p_context->level;
     return VLC_SUCCESS;
 }
 
@@ -594,7 +622,7 @@ static void Flush( decoder_t *p_dec )
     decoder_sys_t *p_sys = p_dec->p_sys;
     AVCodecContext *p_context = p_sys->p_context;
 
-    p_sys->i_pts = VLC_TS_INVALID; /* To make sure we recover properly */
+    date_Set(&p_sys->pts, VLC_TS_INVALID); /* To make sure we recover properly */
     p_sys->i_late_frames = 0;
     cc_Flush( &p_sys->cc );
 
@@ -620,7 +648,7 @@ static bool check_block_validity( decoder_sys_t *p_sys, block_t *block )
 
     if( block->i_flags & (BLOCK_FLAG_DISCONTINUITY|BLOCK_FLAG_CORRUPTED) )
     {
-        p_sys->i_pts = VLC_TS_INVALID; /* To make sure we recover properly */
+        date_Set( &p_sys->pts, VLC_TS_INVALID ); /* To make sure we recover properly */
         cc_Flush( &p_sys->cc );
 
         p_sys->i_late_frames = 0;
@@ -652,10 +680,7 @@ static bool check_block_being_late( decoder_sys_t *p_sys, block_t *block, mtime_
 
     if( current_time - p_sys->i_late_frames_start > (5*CLOCK_FREQ))
     {
-        if( p_sys->i_pts > VLC_TS_INVALID )
-        {
-            p_sys->i_pts = VLC_TS_INVALID; /* To make sure we recover properly */
-        }
+        date_Set( &p_sys->pts, VLC_TS_INVALID ); /* To make sure we recover properly */
         if( block )
             block_Release( block );
         p_sys->i_late_frames--;
@@ -691,27 +716,16 @@ static void interpolate_next_pts( decoder_t *p_dec, AVFrame *frame )
     decoder_sys_t *p_sys = p_dec->p_sys;
     AVCodecContext *p_context = p_sys->p_context;
 
-    if( p_sys->i_pts <= VLC_TS_INVALID )
+    if( date_Get( &p_sys->pts ) == VLC_TS_INVALID ||
+        p_sys->pts.i_divider_num == 0 )
         return;
 
-    /* interpolate the next PTS */
-    if( p_dec->fmt_in.video.i_frame_rate > 0 &&
-        p_dec->fmt_in.video.i_frame_rate_base > 0 )
-    {
-        p_sys->i_pts += CLOCK_FREQ * (2 + frame->repeat_pict) *
-            p_dec->fmt_in.video.i_frame_rate_base /
-            (2 * p_dec->fmt_in.video.i_frame_rate);
-    }
-    else if( p_context->time_base.den > 0 )
-    {
-        int i_tick = p_context->ticks_per_frame;
-        if( i_tick <= 0 )
-            i_tick = 1;
+    int i_tick = p_context->ticks_per_frame;
+    if( i_tick <= 0 )
+        i_tick = 1;
 
-        p_sys->i_pts += CLOCK_FREQ * (2 + frame->repeat_pict) *
-            i_tick * p_context->time_base.num /
-            (2 * p_context->time_base.den);
-    }
+    /* interpolate the next PTS */
+    date_Increment( &p_sys->pts, i_tick + frame->repeat_pict );
 }
 
 static void update_late_frame_count( decoder_t *p_dec, block_t *p_block, mtime_t current_time, mtime_t i_pts )
@@ -747,6 +761,110 @@ static void update_late_frame_count( decoder_t *p_dec, block_t *p_block, mtime_t
 }
 
 
+static void DecodeSidedata( decoder_t *p_dec, const AVFrame *frame, picture_t *p_pic )
+{
+    decoder_sys_t *p_sys = p_dec->p_sys;
+    bool format_changed = false;
+
+#if (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT( 55, 16, 101 ) )
+#define FROM_AVRAT(default_factor, avrat) \
+(uint64_t)(default_factor) * (avrat).num / (avrat).den
+    const AVFrameSideData *metadata =
+            av_frame_get_side_data( frame,
+                                    AV_FRAME_DATA_MASTERING_DISPLAY_METADATA );
+    if ( metadata )
+    {
+        const AVMasteringDisplayMetadata *hdr_meta =
+                (const AVMasteringDisplayMetadata *) metadata->data;
+        if ( hdr_meta->has_luminance )
+        {
+#define ST2086_LUMA_FACTOR 10000
+            p_pic->format.mastering.max_luminance =
+                    FROM_AVRAT(ST2086_LUMA_FACTOR, hdr_meta->max_luminance);
+            p_pic->format.mastering.min_luminance =
+                    FROM_AVRAT(ST2086_LUMA_FACTOR, hdr_meta->min_luminance);
+        }
+        if ( hdr_meta->has_primaries )
+        {
+#define ST2086_RED   2
+#define ST2086_GREEN 0
+#define ST2086_BLUE  1
+#define LAV_RED    0
+#define LAV_GREEN  1
+#define LAV_BLUE   2
+#define ST2086_PRIM_FACTOR 50000
+            p_pic->format.mastering.primaries[ST2086_RED*2   + 0] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_RED][0]);
+            p_pic->format.mastering.primaries[ST2086_RED*2   + 1] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_RED][1]);
+            p_pic->format.mastering.primaries[ST2086_GREEN*2 + 0] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_GREEN][0]);
+            p_pic->format.mastering.primaries[ST2086_GREEN*2 + 1] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_GREEN][1]);
+            p_pic->format.mastering.primaries[ST2086_BLUE*2  + 0] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_BLUE][0]);
+            p_pic->format.mastering.primaries[ST2086_BLUE*2  + 1] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_BLUE][1]);
+            p_pic->format.mastering.white_point[0] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->white_point[0]);
+            p_pic->format.mastering.white_point[1] =
+                    FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->white_point[1]);
+        }
+
+        if ( memcmp( &p_dec->fmt_out.video.mastering,
+                     &p_pic->format.mastering,
+                     sizeof(p_pic->format.mastering) ) )
+        {
+            p_dec->fmt_out.video.mastering = p_pic->format.mastering;
+            format_changed = true;
+        }
+#undef FROM_AVRAT
+    }
+#endif
+#if (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT( 55, 60, 100 ) )
+    const AVFrameSideData *metadata_lt =
+            av_frame_get_side_data( frame,
+                                    AV_FRAME_DATA_CONTENT_LIGHT_LEVEL );
+    if ( metadata_lt )
+    {
+        const AVContentLightMetadata *light_meta =
+                (const AVContentLightMetadata *) metadata_lt->data;
+        p_pic->format.lighting.MaxCLL = light_meta->MaxCLL;
+        p_pic->format.lighting.MaxFALL = light_meta->MaxFALL;
+        if ( memcmp( &p_dec->fmt_out.video.lighting,
+                     &p_pic->format.lighting,
+                     sizeof(p_pic->format.lighting) ) )
+        {
+            p_dec->fmt_out.video.lighting  = p_pic->format.lighting;
+            format_changed = true;
+        }
+    }
+#endif
+
+    if (format_changed)
+        decoder_UpdateVideoFormat( p_dec );
+
+    const AVFrameSideData *p_avcc = av_frame_get_side_data( frame, AV_FRAME_DATA_A53_CC );
+    if( p_avcc )
+    {
+        cc_Extract( &p_sys->cc, CC_PAYLOAD_RAW, true, p_avcc->data, p_avcc->size );
+        if( p_sys->cc.i_data )
+        {
+            block_t *p_cc = block_Alloc( p_sys->cc.i_data );
+            if( p_cc )
+            {
+                memcpy( p_cc->p_buffer, p_sys->cc.p_data, p_sys->cc.i_data );
+                if( p_sys->cc.b_reorder )
+                    p_cc->i_dts = p_cc->i_pts = p_pic->date;
+                else
+                    p_cc->i_pts = p_cc->i_dts;
+                decoder_QueueCc( p_dec, p_cc, p_sys->cc.pb_present, 4 );
+            }
+            cc_Flush( &p_sys->cc );
+        }
+    }
+}
+
 /*****************************************************************************
  * DecodeBlock: Called to decode one or more frames
  *****************************************************************************/
@@ -767,7 +885,7 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
     if( !p_context->extradata_size && p_dec->fmt_in.i_extra )
     {
         ffmpeg_InitCodec( p_dec );
-        if( p_sys->b_delayed_open )
+        if( !avcodec_is_open( p_context ) )
             OpenVideoCodec( p_dec );
     }
 
@@ -775,7 +893,7 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
     if(!p_block && !(p_sys->p_codec->capabilities & AV_CODEC_CAP_DELAY) )
         return NULL;
 
-    if( p_sys->b_delayed_open )
+    if( !avcodec_is_open( p_context ) )
     {
         if( p_block )
             block_Release( p_block );
@@ -852,7 +970,7 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
         post_mt( p_sys );
 
         av_init_packet( &pkt );
-        if( p_block )
+        if( p_block && p_block->i_buffer > 0 )
         {
             pkt.data = p_block->p_buffer;
             pkt.size = p_block->i_buffer;
@@ -917,6 +1035,9 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
                 *error = true;
             }
             av_frame_free(&frame);
+            /* After draining, we need to reset decoder with a flush */
+            if( ret == AVERROR_EOF )
+                avcodec_flush_buffers( p_sys->p_context );
             break;
         }
         bool not_received_frame = ret;
@@ -954,11 +1075,11 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
             i_pts = frame->pkt_dts;
 
         if( i_pts == AV_NOPTS_VALUE )
-            i_pts = p_sys->i_pts;
+            i_pts = date_Get( &p_sys->pts );
 
         /* Interpolate the next PTS */
         if( i_pts > VLC_TS_INVALID )
-            p_sys->i_pts = i_pts;
+            date_Set( &p_sys->pts, i_pts );
 
         interpolate_next_pts( p_dec, frame );
 
@@ -1037,8 +1158,6 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
         }
         else
         {
-            if( p_sys->p_va != NULL )
-                vlc_va_Extract( p_sys->p_va, p_pic, frame->data[3] );
             picture_Hold( p_pic );
         }
 
@@ -1064,107 +1183,10 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block, bool *error
         p_pic->b_progressive = !frame->interlaced_frame;
         p_pic->b_top_field_first = frame->top_field_first;
 
-        bool format_changed = false;
-#if (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT( 55, 16, 101 ) )
-#define FROM_AVRAT(default_factor, avrat) \
-    (uint64_t)(default_factor) * (avrat).num / (avrat).den
-        const AVFrameSideData *metadata =
-                av_frame_get_side_data( frame,
-                                        AV_FRAME_DATA_MASTERING_DISPLAY_METADATA );
-        if ( metadata )
-        {
-            const AVMasteringDisplayMetadata *hdr_meta =
-                    (const AVMasteringDisplayMetadata *) metadata->data;
-            if ( hdr_meta->has_luminance )
-            {
-#define ST2086_LUMA_FACTOR 10000
-                p_pic->format.mastering.max_luminance =
-                        FROM_AVRAT(ST2086_LUMA_FACTOR, hdr_meta->max_luminance);
-                p_pic->format.mastering.min_luminance =
-                        FROM_AVRAT(ST2086_LUMA_FACTOR, hdr_meta->min_luminance);
-            }
-            if ( hdr_meta->has_primaries )
-            {
-#define ST2086_RED   2
-#define ST2086_GREEN 0
-#define ST2086_BLUE  1
-#define LAV_RED    0
-#define LAV_GREEN  1
-#define LAV_BLUE   2
-#define ST2086_PRIM_FACTOR 50000
-                p_pic->format.mastering.primaries[ST2086_RED*2   + 0] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_RED][0]);
-                p_pic->format.mastering.primaries[ST2086_RED*2   + 1] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_RED][1]);
-                p_pic->format.mastering.primaries[ST2086_GREEN*2 + 0] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_GREEN][0]);
-                p_pic->format.mastering.primaries[ST2086_GREEN*2 + 1] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_GREEN][1]);
-                p_pic->format.mastering.primaries[ST2086_BLUE*2  + 0] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_BLUE][0]);
-                p_pic->format.mastering.primaries[ST2086_BLUE*2  + 1] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->display_primaries[LAV_BLUE][1]);
-                p_pic->format.mastering.white_point[0] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->white_point[0]);
-                p_pic->format.mastering.white_point[1] =
-                        FROM_AVRAT(ST2086_PRIM_FACTOR, hdr_meta->white_point[1]);
-            }
-
-            if ( memcmp( &p_dec->fmt_out.video.mastering,
-                         &p_pic->format.mastering,
-                         sizeof(p_pic->format.mastering) ) )
-            {
-                p_dec->fmt_out.video.mastering = p_pic->format.mastering;
-                format_changed = true;
-            }
-#undef FROM_AVRAT
-        }
-#endif
-#if (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT( 55, 60, 100 ) )
-        const AVFrameSideData *metadata_lt =
-                av_frame_get_side_data( frame,
-                                        AV_FRAME_DATA_CONTENT_LIGHT_LEVEL );
-        if ( metadata_lt )
-        {
-            const AVContentLightMetadata *light_meta =
-                    (const AVContentLightMetadata *) metadata_lt->data;
-            p_pic->format.lighting.MaxCLL = light_meta->MaxCLL;
-            p_pic->format.lighting.MaxFALL = light_meta->MaxFALL;
-            if ( memcmp( &p_dec->fmt_out.video.lighting,
-                         &p_pic->format.lighting,
-                         sizeof(p_pic->format.lighting) ) )
-            {
-                p_dec->fmt_out.video.lighting  = p_pic->format.lighting;
-                format_changed = true;
-            }
-        }
-#endif
-
-        const AVFrameSideData *p_avcc = av_frame_get_side_data( frame, AV_FRAME_DATA_A53_CC );
-        if( p_avcc )
-        {
-            cc_Extract( &p_sys->cc, CC_PAYLOAD_RAW, true, p_avcc->data, p_avcc->size );
-            if( p_sys->cc.i_data )
-            {
-                block_t *p_cc = block_Alloc( p_sys->cc.i_data );
-                if( p_cc )
-                {
-                    memcpy( p_cc->p_buffer, p_sys->cc.p_data, p_sys->cc.i_data );
-                    if( p_sys->cc.b_reorder )
-                        p_cc->i_dts = p_cc->i_pts = i_pts;
-                    else
-                        p_cc->i_pts = p_cc->i_dts;
-                    decoder_QueueCc( p_dec, p_cc, p_sys->cc.pb_present, 4 );
-                }
-                cc_Flush( &p_sys->cc );
-            }
-        }
+        DecodeSidedata( p_dec, frame, p_pic );
 
         av_frame_free(&frame);
 
-        if (format_changed)
-            decoder_UpdateVideoFormat( p_dec );
-
         /* Send decoded frame to vout */
         if (i_pts > VLC_TS_INVALID)
         {
@@ -1196,26 +1218,31 @@ static int DecodeVideo( decoder_t *p_dec, block_t *p_block )
  * This function is called when the thread ends after a successful
  * initialization.
  *****************************************************************************/
-void EndVideoDec( decoder_t *p_dec )
+void EndVideoDec( vlc_object_t *obj )
 {
+    decoder_t *p_dec = (decoder_t *)obj;
     decoder_sys_t *p_sys = p_dec->p_sys;
+    AVCodecContext *ctx = p_sys->p_context;
+    void *hwaccel_context;
 
     post_mt( p_sys );
 
     /* do not flush buffers if codec hasn't been opened (theora/vorbis/VC1) */
-    if( avcodec_is_open( p_sys->p_context ) )
-        avcodec_flush_buffers( p_sys->p_context );
+    if( avcodec_is_open( ctx ) )
+        avcodec_flush_buffers( ctx );
 
     wait_mt( p_sys );
 
     cc_Flush( &p_sys->cc );
 
-    ffmpeg_CloseCodec( p_dec );
+    hwaccel_context = ctx->hwaccel_context;
+    avcodec_free_context( &ctx );
 
     if( p_sys->p_va )
-        vlc_va_Delete( p_sys->p_va, p_sys->p_context );
+        vlc_va_Delete( p_sys->p_va, &hwaccel_context );
 
     vlc_sem_destroy( &p_sys->sem_mt );
+    free( p_sys );
 }
 
 /*****************************************************************************
@@ -1308,14 +1335,10 @@ static int lavc_va_GetFrame(struct AVCodecContext *ctx, AVFrame *frame,
      * data[3] actually contains the format-specific surface handle. */
     frame->data[3] = frame->data[0];
 
-    void (*release)(void *, uint8_t *) = va->release;
-    if (va->release == NULL)
-        release = lavc_ReleaseFrame;
-
-    frame->buf[0] = av_buffer_create(frame->data[0], 0, release, pic, 0);
+    frame->buf[0] = av_buffer_create(frame->data[0], 0, lavc_ReleaseFrame, pic, 0);
     if (unlikely(frame->buf[0] == NULL))
     {
-        release(pic, frame->data[0]);
+        lavc_ReleaseFrame(pic, frame->data[0]);
         return -1;
     }
 
@@ -1491,7 +1514,7 @@ static enum PixelFormat ffmpeg_GetFormat( AVCodecContext *p_context,
     if (p_sys->p_va != NULL)
     {
         msg_Err(p_dec, "existing hardware acceleration cannot be reused");
-        vlc_va_Delete(p_sys->p_va, p_context);
+        vlc_va_Delete(p_sys->p_va, &p_context->hwaccel_context);
         p_sys->p_va = NULL;
     }
 
