@@ -88,7 +88,7 @@ vlc_module_begin ()
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_VCODEC )
     set_description( N_("Crystal HD hardware video decoder") )
-    set_capability( "decoder", 0 )
+    set_capability( "video decoder", 0 )
     set_callbacks( OpenDecoder, CloseDecoder )
     add_shortcut( "crystalhd" )
 vlc_module_end ()
