@@ -2,7 +2,7 @@
  * dmo.c : DirectMedia Object decoder module for vlc
  *****************************************************************************
  * Copyright (C) 2002, 2003 VLC authors and VideoLAN
- * $Id: ef40f3859f4fffaab89ce9a815cbd666feac3a19 $
+ * $Id: 86d625e92b30d66bdc3751ca72a7bd4d7336c77f $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -90,11 +90,16 @@ static void CopyPicture( picture_t *, uint8_t * );
 vlc_module_begin ()
     set_description( N_("DirectMedia Object decoder") )
     add_shortcut( "dmo" )
-    set_capability( "decoder", 1 )
+    set_capability( "video decoder", 1 )
     set_callbacks( DecoderOpen, DecoderClose )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_VCODEC )
 
+    add_submodule()
+    add_shortcut("dmo")
+    set_capability( "audio decoder", 1 )
+    set_callbacks(DecoderOpen, DecoderClose)
+
 #   define ENC_CFG_PREFIX "sout-dmo-"
     add_submodule ()
     set_description( N_("DirectMedia Object encoder") )
