@@ -2,7 +2,7 @@
  * faad.c: AAC decoder using libfaad2
  *****************************************************************************
  * Copyright (C) 2001, 2003 VLC authors and VideoLAN
- * $Id: 809983fd52eee94b77ed231b5664e776c161a576 $
+ * $Id: ec9e2abb6b060c82613c0314e0f757dd8b608ab1 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -52,7 +52,7 @@ static void Close( vlc_object_t * );
 
 vlc_module_begin ()
     set_description( N_("AAC audio decoder (using libfaad2)") )
-    set_capability( "decoder", 100 )
+    set_capability( "audio decoder", 100 )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_ACODEC )
     set_callbacks( Open, Close )
