@@ -2,7 +2,7 @@
  * flac.c: flac decoder/encoder module making use of libflac
  *****************************************************************************
  * Copyright (C) 1999-2001 VLC authors and VideoLAN
- * $Id: 7d2913a40d63ac7bf0fd0215db632a607ae0a93b $
+ * $Id: 7a831cca7fc8ce5e47ac67eee9212bc047c327b3 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Sigmund Augdal Helberg <dnumgis@videolan.org>
@@ -162,7 +162,7 @@ vlc_module_begin ()
     add_shortcut( "flac" )
 
     set_description( N_("Flac audio decoder") )
-    set_capability( "decoder", 100 )
+    set_capability( "audio decoder", 100 )
     set_callbacks( OpenDecoder, CloseDecoder )
 
 #ifdef ENABLE_SOUT
