@@ -2,7 +2,7 @@
  * g711.c: G.711 µ-law and A-law codec
  *****************************************************************************
  * Copyright (C) 2001, 2003 VLC authors and VideoLAN
- * $Id: 95483e80262e99d2a25ff2f5f06046b2266025e0 $
+ * $Id: e938efb637daacbb682f9919da330df3f2c306b4 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -44,7 +44,7 @@ static block_t *EncoderEncode( encoder_t *, block_t * );
 
 vlc_module_begin ()
     set_description( N_("G.711 decoder") )
-    set_capability( "decoder", 100 )
+    set_capability( "audio decoder", 100 )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_ACODEC )
     set_callbacks( DecoderOpen, DecoderClose )
