@@ -2,7 +2,7 @@
  * SSA/ASS subtitle decoder using libass.
  *****************************************************************************
  * Copyright (C) 2008-2009 VLC authors and VideoLAN
- * $Id: 7d7f265f58dbf9da00741a2d3dfefae59393d8af $
+ * $Id: de53a618ef35f144a918a56676e23c5b59ae21c3 $
  *
  * Authors: Laurent Aimar <fenrir@videolan.org>
  *
@@ -55,7 +55,7 @@ static void Destroy( vlc_object_t * );
 vlc_module_begin ()
     set_shortname( N_("Subtitles (advanced)"))
     set_description( N_("Subtitle renderers using libass") )
-    set_capability( "decoder", 100 )
+    set_capability( "spu decoder", 100 )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_SCODEC )
     set_callbacks( Create, Destroy )
