@@ -2,7 +2,7 @@
  * omxil.c: Video decoder module making use of OpenMAX IL components.
  *****************************************************************************
  * Copyright (C) 2010 VLC authors and VideoLAN
- * $Id: 2ae9bb1874c4a06d0233d547df99169891fc3842 $
+ * $Id: 4451c6efb975d0bf7b9c799678a59163dd9490ae $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -150,13 +150,22 @@ vlc_module_begin ()
     /* For IOMX, don't enable it automatically via priorities,
      * enable it only via the --codec iomx command line parameter when
      * wanted. */
-    set_capability( "decoder", 0 )
+    set_capability( "video decoder", 0 )
     add_bool(CFG_PREFIX "dr", true,
              DIRECTRENDERING_TEXT, DIRECTRENDERING_LONGTEXT, true)
 #else
-    set_capability( "decoder", 80 )
+    set_capability( "video decoder", 80 )
 #endif
     set_callbacks( OpenDecoder, CloseGeneric )
+#ifndef __ANDROID__
+    add_submodule()
+# if defined(USE_IOMX)
+    set_capability("audio decoder", 0)
+# else
+    set_capability("audio decoder", 80)
+# endif
+    set_callbacks(OpenDecoder, CloseGeneric)
+#endif
 
     add_submodule ()
     set_section( N_("Encoding") , NULL )
@@ -1008,11 +1017,6 @@ static int OpenDecoder( vlc_object_t *p_this )
     decoder_t *p_dec = (decoder_t*)p_this;
     int status;
 
-#ifdef __ANDROID__
-    if( p_dec->fmt_in.i_cat == AUDIO_ES )
-        return VLC_EGENERIC;
-#endif
-
     if( 0 || !GetOmxRole(p_dec->fmt_in.i_codec, p_dec->fmt_in.i_cat, false) )
         return VLC_EGENERIC;
 
