@@ -34,7 +34,7 @@ vlc_module_begin()
     set_category(CAT_INPUT)
     set_subcategory(SUBCAT_INPUT_ACODEC)
     set_description(N_("S/PDIF pass-through decoder"))
-    set_capability("decoder", 120)
+    set_capability("audio decoder", 120)
     set_callbacks(OpenDecoder, NULL)
 vlc_module_end()
 
