@@ -2,7 +2,7 @@
  * speex.c: speex decoder/packetizer/encoder module making use of libspeex.
  *****************************************************************************
  * Copyright (C) 2003-2009 VLC authors and VideoLAN
- * $Id: 6cec5696d41b7d6d41bc411c0c53e3d5d5409309 $
+ * $Id: 9ada14c7a1c8831bee4cd7b217320ac3399a0002 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -96,7 +96,7 @@ vlc_module_begin ()
     set_subcategory( SUBCAT_INPUT_ACODEC )
 
     set_description( N_("Speex audio decoder") )
-    set_capability( "decoder", 100 )
+    set_capability( "audio decoder", 100 )
     set_shortname( N_("Speex") )
     set_callbacks( OpenDecoder, CloseDecoder )
 
