@@ -2,7 +2,7 @@
  * stl.c: EBU STL decoder
  *****************************************************************************
  * Copyright (C) 2010 Laurent Aimar
- * $Id: 1875678e37249a0b0fa2bdd7ddd742551412c511 $
+ * $Id: 38b60e1f7e0ee2c482e441ebf9ee76f621992f84 $
  *
  * Authors: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
@@ -47,7 +47,7 @@ vlc_module_begin()
     set_description(N_("EBU STL subtitles decoder"))
     set_category(CAT_INPUT)
     set_subcategory(SUBCAT_INPUT_SCODEC)
-    set_capability("decoder", 10)
+    set_capability("spu decoder", 10)
     set_callbacks(Open, Close)
 vlc_module_end()
 
