@@ -2,7 +2,7 @@
  * subsusf.c : USF subtitles decoder
  *****************************************************************************
  * Copyright (C) 2000-2006 VLC authors and VideoLAN
- * $Id: 1a94b6b4fdfb2deeabb4e6d6e193e1b6fa95ed27 $
+ * $Id: 14b07e051a0cf46c366a26a079784fe987bae0e0 $
  *
  * Authors: Bernie Purcell <bitmap@videolan.org>
  *
@@ -46,7 +46,7 @@ static void CloseDecoder  ( vlc_object_t * );
  "VLC partly implements this, but you can choose to disable all formatting.")
 
 vlc_module_begin ()
-    set_capability( "decoder", 40 )
+    set_capability( "spu decoder", 40 )
     set_shortname( N_("USFSubs"))
     set_description( N_("USF subtitles decoder") )
     set_callbacks( OpenDecoder, CloseDecoder )
