@@ -5,7 +5,7 @@
  * Copyright (C) 2007 Société des arts technologiques
  * Copyright (C) 2007 Savoir-faire Linux
  *
- * $Id: 6f6ed8ba54ac39e7f3b126a2ef3f958ac2fe4019 $
+ * $Id: 4623e38a1edc1b2cec4e34981c62baa581ec2c1a $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -188,9 +188,9 @@ vlc_module_begin ()
     set_shortname( "Vorbis" )
     set_description( N_("Vorbis audio decoder") )
 #ifdef MODULE_NAME_IS_tremor
-    set_capability( "decoder", 90 )
+    set_capability( "audio decoder", 90 )
 #else
-    set_capability( "decoder", 100 )
+    set_capability( "audio decoder", 100 )
 #endif
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_ACODEC )
