@@ -2,7 +2,7 @@
  * demux.c: demuxer using libavformat
  *****************************************************************************
  * Copyright (C) 2004-2009 VLC authors and VideoLAN
- * $Id: 1dd0976ee340fd9fce86eee8d6a90e107c06221b $
+ * $Id: 7b00c781125613bfb0872c44768ee3912a58b2b7 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -373,10 +373,9 @@ int OpenDemux( vlc_object_t *p_this )
         const AVCodecParameters *cp = s->codecpar;
         es_out_id_t  *es = NULL;
         es_format_t  fmt;
-        vlc_fourcc_t fcc;
         const char *psz_type = "unknown";
-
-        if( !GetVlcFourcc( cp->codec_id, NULL, &fcc, NULL ) )
+        vlc_fourcc_t fcc = GetVlcFourcc( cp->codec_id );
+        if( !fcc )
             fcc = VLC_FOURCC( 'u', 'n', 'd', 'f' );
 
 #if LIBAVFORMAT_VERSION_INT >= ((54<<16)+(2<<8)+0)
