@@ -2,7 +2,7 @@
  * m3u.c : M3U playlist format import
  *****************************************************************************
  * Copyright (C) 2004 VLC authors and VideoLAN
- * $Id: 651be26440443e4cfb632e5556089544ebcd4d3f $
+ * $Id: ecd71b9e4e4e3663b9ea712eb3e542e3a99f611d $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Sigmund Augdal Helberg <dnumgis@videolan.org>
@@ -31,23 +31,18 @@
 #endif
 
 #include <vlc_common.h>
-#include <vlc_demux.h>
+#include <vlc_access.h>
 #include <vlc_charset.h>
+#include <vlc_strings.h>
 
 #include "playlist.h"
 
-struct demux_sys_t
-{
-    char *psz_prefix;
-    char *(*pf_dup) (const char *);
-};
-
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int Demux( demux_t *p_demux);
+static int ReadDir( stream_t *, input_item_node_t * );
 static void parseEXTINF( char *psz_string, char **ppsz_artist, char **ppsz_name, int *pi_duration );
-static bool ContainsURL( demux_t *p_demux );
+static bool ContainsURL(const uint8_t *, size_t);
 
 static char *GuessEncoding (const char *str)
 {
@@ -64,70 +59,98 @@ static char *CheckUnicode (const char *str)
  *****************************************************************************/
 int Import_M3U( vlc_object_t *p_this )
 {
-    demux_t *p_demux = (demux_t *)p_this;
+    stream_t *p_stream = (stream_t *)p_this;
     const uint8_t *p_peek;
+    ssize_t i_peek;
     char *(*pf_dup) (const char *) = GuessEncoding;
     int offset = 0;
 
-    CHECK_FILE();
-    if( vlc_stream_Peek( p_demux->s, &p_peek, 3 ) == 3
-     && !memcmp( p_peek, "\xef\xbb\xbf", 3) )
+    CHECK_FILE(p_stream);
+    i_peek = vlc_stream_Peek( p_stream->p_source, &p_peek, 1024 );
+    if( i_peek < 8 )
+        return VLC_EGENERIC;
+
+    if( !memcmp( p_peek, "\xef\xbb\xbf", 3) )
     {
+        if( i_peek < 12 )
+            return VLC_EGENERIC;
         pf_dup = CheckUnicode; /* UTF-8 Byte Order Mark */
         offset = 3;
     }
 
-    char *type = stream_MimeType(p_demux->s);
-
-    if( demux_IsPathExtension( p_demux, ".m3u8" )
-     || demux_IsForced( p_demux, "m3u8" )
-     || (type != NULL && !strcasecmp(type, "application/vnd.apple.mpegurl")))
+    char *type = stream_MimeType(p_stream->p_source);
+    if (type != NULL
+     && (!vlc_ascii_strcasecmp(type, "application/vnd.apple.mpegurl")
+      || !vlc_ascii_strcasecmp(type, "audio/mpegurl")))
     {
-        pf_dup = CheckUnicode; /* UTF-8 file type */
         free(type);
+        return VLC_EGENERIC;
     }
-    else
-    if( demux_IsPathExtension( p_demux, ".m3u" )
-     || demux_IsPathExtension( p_demux, ".vlc" )
-     || demux_IsForced( p_demux, "m3u" )
-     || ContainsURL( p_demux )
-     || (type != NULL && !strcasecmp(type, "audio/x-mpegurl")))
-        free(type); /* Guess encoding */
-    else
-    {
-        free(type);
 
-        if( vlc_stream_Peek( p_demux->s, &p_peek, 8 + offset ) < (8 + offset) )
-            return VLC_EGENERIC;
+    bool b_check_hls = true;
 
+    if( stream_HasExtension( p_stream, ".m3u8" ) )
+        pf_dup = CheckUnicode; /* UTF-8 file type */
+    else
+    if (stream_HasExtension(p_stream, ".vlc"))
+        b_check_hls = false;
+    else
+    if (stream_HasExtension(p_stream, ".m3u")
+     || p_stream->obj.force
+     || (type != NULL
+      && (!vlc_ascii_strcasecmp(type, "application/mpegurl")
+       || !vlc_ascii_strcasecmp(type, "application/x-mpegurl")
+       || !vlc_ascii_strcasecmp(type, "audio/mpegurl")
+       || !vlc_ascii_strcasecmp(type, "audio/x-mpegurl")))
+     || ContainsURL(p_peek, i_peek))
+        ;
+    else
+    {   /* Guess encoding */
         p_peek += offset;
 
         if( !strncasecmp( (const char *)p_peek, "RTSPtext", 8 ) ) /* QuickTime */
+        {
             pf_dup = CheckUnicode; /* UTF-8 */
-        else
-        if( !memcmp( p_peek, "#EXTM3U", 7 ) )
-            ; /* Guess encoding */
-        else
+            b_check_hls = false;
+        }
+        else if( memcmp( p_peek, "#EXTM3U", 7 ) )
+        {
+            free( type );
             return VLC_EGENERIC;
+        }
     }
 
-    vlc_stream_Seek( p_demux->s, offset );
+    free(type);
 
-    STANDARD_DEMUX_INIT_MSG( "found valid M3U playlist" );
-    p_demux->p_sys->psz_prefix = FindPrefix( p_demux );
-    p_demux->p_sys->pf_dup = pf_dup;
+    if( b_check_hls )
+    {
+        const char * ppsz_hlsexts[] =
+        {
+            "#EXT-X-MEDIA:",
+            "#EXT-X-VERSION:",
+            "#EXT-X-TARGETDURATION:",
+            "#EXT-X-MEDIA-SEQUENCE:",
+        };
+        for( size_t i=0; i<ARRAY_SIZE(ppsz_hlsexts); i++ )
+            if( strnstr( (const char *) p_peek, ppsz_hlsexts[i], i_peek - offset ) )
+                return VLC_EGENERIC;
+    }
+
+    vlc_stream_Seek( p_stream->p_source, offset );
+
+    msg_Dbg( p_stream, "found valid M3U playlist" );
+    p_stream->p_sys = pf_dup;
+    p_stream->pf_readdir = ReadDir;
+    p_stream->pf_control = access_vaDirectoryControlHelper;
 
     return VLC_SUCCESS;
 }
 
-static bool ContainsURL( demux_t *p_demux )
+static bool ContainsURL(const uint8_t *p_peek, size_t i_peek)
 {
-    const uint8_t *p_peek, *p_peek_end;
-    int i_peek;
-
-    i_peek = vlc_stream_Peek( p_demux->s, &p_peek, 1024 );
-    if( i_peek <= 0 ) return false;
-    p_peek_end = p_peek + i_peek;
+    const uint8_t *p_peek_end = p_peek + i_peek;
+    if( i_peek < 7 )
+        return false;
 
     while( p_peek + sizeof( "https://" ) < p_peek_end )
     {
@@ -156,18 +179,7 @@ static bool ContainsURL( demux_t *p_demux )
     return false;
 }
 
-/*****************************************************************************
- * Deactivate: frees unused data
- *****************************************************************************/
-void Close_M3U( vlc_object_t *p_this )
-{
-    demux_t *p_demux = (demux_t *)p_this;
-    free( p_demux->p_sys->psz_prefix );
-    free( p_demux->p_sys );
-}
-
-
-static int Demux( demux_t *p_demux )
+static int ReadDir( stream_t *p_demux, input_item_node_t *p_subitems )
 {
     char       *psz_line;
     char       *psz_name = NULL;
@@ -176,15 +188,14 @@ static int Demux( demux_t *p_demux )
     int        i_parsed_duration = 0;
     mtime_t    i_duration = -1;
     const char**ppsz_options = NULL;
-    char *    (*pf_dup) (const char *) = p_demux->p_sys->pf_dup;
+    char *    (*pf_dup) (const char *) = p_demux->p_sys;
     int        i_options = 0;
     bool b_cleanup = false;
     input_item_t *p_input;
 
     input_item_t *p_current_input = GetCurrentItem(p_demux);
-    input_item_node_t *p_subitems = input_item_node_Create( p_current_input );
 
-    psz_line = vlc_stream_ReadLine( p_demux->s );
+    psz_line = vlc_stream_ReadLine( p_demux->p_source );
     while( psz_line )
     {
         char *psz_parse = psz_line;
@@ -252,7 +263,7 @@ static int Demux( demux_t *p_demux )
                 /* Use filename as name for relative entries */
                 psz_name = strdup( psz_parse );
 
-            psz_mrl = ProcessMRL( psz_parse, p_demux->p_sys->psz_prefix );
+            psz_mrl = ProcessMRL( psz_parse, p_demux->psz_url );
 
             b_cleanup = true;
             if( !psz_mrl )
@@ -285,7 +296,7 @@ static int Demux( demux_t *p_demux )
 
         /* Fetch another line */
         free( psz_line );
-        psz_line = vlc_stream_ReadLine( p_demux->s );
+        psz_line = vlc_stream_ReadLine( p_demux->p_source );
         if( !psz_line ) b_cleanup = true;
 
         if( b_cleanup )
@@ -303,9 +314,7 @@ static int Demux( demux_t *p_demux )
             b_cleanup = false;
         }
     }
-    input_item_node_PostAndDelete( p_subitems );
-    var_Destroy( p_demux, "m3u-extvlcopt" );
-    return 0; /* Needed for correct operation of go back */
+    return VLC_SUCCESS; /* Needed for correct operation of go back */
 }
 
 static void parseEXTINF(char *psz_string, char **ppsz_artist,
