@@ -2,7 +2,7 @@
  * subtitle.c: Demux for subtitle text files.
  *****************************************************************************
  * Copyright (C) 1999-2007 VLC authors and VideoLAN
- * $Id: 9726bc26bb61c0062f4a8b27211ba5f7fdf8045b $
+ * $Id: d2f3e9d0b4e6165bdc0b345b89b7c59654ace9e5 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Derk-Jan Hartman <hartman at videolan dot org>
@@ -1260,11 +1260,9 @@ static char *ParseSamiSearch( text_t *txt,
         if( !p )
             return NULL;
 
-        if( strcasestr( p, psz_str ) )
-        {
-            char *s = strcasestr( p, psz_str );
+        char *s = strcasestr( p, psz_str );
+        if( s != NULL )
             return &s[strlen( psz_str )];
-        }
     }
 }
 static int ParseSami( vlc_object_t *p_obj, subs_properties_t *p_props,
