@@ -2,7 +2,7 @@
  * VLCControlsBarCommon.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2012-2016 VLC authors and VideoLAN
- * $Id: 35dc8321e794b0c2eb90399b22a42fa924e7d0c0 $
+ * $Id: 642a1cce7313efb9d49fc25b8f4c7474861295ad $
  *
  * Authors: Felix Paul Kühne <fkuehne -at- videolan -dot- org>
  *          David Fuhrmann <david dot fuhrmann at googlemail dot com>
@@ -144,13 +144,6 @@
     frame.size.width = 0;
     [_timeSliderGradientView setFrame: frame];
 
-    // hide resize view if necessary
-    [self.resizeView setImage: NULL];
-
-    if ([[self.bottomBarView window] styleMask] & NSResizableWindowMask)
-        [self.resizeView removeFromSuperviewWithoutNeedingDisplay];
-
-
     // remove fullscreen button for lion fullscreen
     if (_nativeFullscreenMode) {
         [self.fullscreenButton removeFromSuperview];
