@@ -2,7 +2,7 @@
  * VLCMainWindow.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2002-2013 VLC authors and VideoLAN
- * $Id: bb960e82b89965475e70b8525d71f10cef04f8c0 $
+ * $Id: 5fed2473fd370ff116ed39c8f42a36f5eded692d $
  *
  * Authors: Felix Paul Kühne <fkuehne -at- videolan -dot- org>
  *          Jon Lech Johansen <jon-vl@nanocrew.net>
@@ -75,7 +75,6 @@
 
     NSRect frameBeforePlayback;
 }
-- (void)resizePlaylistAfterCollapse;
 - (void)makeSplitViewVisible;
 - (void)makeSplitViewHidden;
 - (void)showPodcastControls;
@@ -176,9 +175,6 @@ static const float f_min_window_height = 307.;
     [self reloadSidebar];
     [_sidebarView selectRowIndexes:[NSIndexSet indexSetWithIndex:1] byExtendingSelection:NO];
 
-    // Setup view frame sizes
-    [_dropzoneView setFrame:_playlistScrollView.frame];
-    [_splitViewLeft setFrame:_sidebarView.frame];
 
     /*
      * Set up translatable strings for the UI elements
@@ -229,13 +225,6 @@ static const float f_min_window_height = 307.;
     else
         [self setContentMinSize:NSMakeSize(604., f_min_window_height)];
 
-    /* the default small size of the search field is slightly different on Lion, let's work-around that */
-    NSRect frame;
-    frame = [_searchField frame];
-    frame.origin.y = frame.origin.y + 2.0;
-    frame.size.height = frame.size.height - 1.0;
-    [_searchField setFrame:frame];
-
     _fspanel = [[VLCFSPanelController alloc] init];
     [_fspanel showWindow:self];
 
@@ -266,7 +255,6 @@ static const float f_min_window_height = 307.;
         [[self contentView] addSubview:o_color_backdrop positioned:NSWindowBelow relativeTo:_splitView];
         [o_color_backdrop setAutoresizingMask:NSViewHeightSizable | NSViewWidthSizable];
     } else {
-        [self.videoView setFrame:_splitView.frame];
         [_playlistScrollView setBorderType:NSNoBorder];
         [_sidebarScrollView setBorderType:NSNoBorder];
     }
@@ -282,7 +270,7 @@ static const float f_min_window_height = 307.;
     }
 
     /* sanity check for the window size */
-    frame = [self frame];
+    NSRect frame = [self frame];
     NSSize screenSize = [[self screen] frame].size;
     if (screenSize.width <= frame.size.width || screenSize.height <= frame.size.height) {
         self.nativeVideoSize = screenSize;
@@ -410,33 +398,7 @@ static const float f_min_window_height = 307.;
     }
 }
 
-- (void)resizePlaylistAfterCollapse
-{
-    // no animation here since we might be in the middle of another resize animation
-    NSRect rightSplitRect = [_splitViewRight frame];
-
-    NSRect plrect;
-    plrect.size.height = rightSplitRect.size.height - 20.0; // actual pl top bar height, which differs from its frame
-    plrect.size.width = rightSplitRect.size.width;
-    plrect.origin.x = plrect.origin.y = 0.;
-
-    NSRect dropzoneboxRect = _dropzoneBox.frame;
-    dropzoneboxRect.origin.x = (plrect.size.width - dropzoneboxRect.size.width) / 2;
-    dropzoneboxRect.origin.y = (plrect.size.height - dropzoneboxRect.size.height) / 2;
-
-    [_dropzoneView setFrame:plrect];
-    [_dropzoneBox setFrame:dropzoneboxRect];
-
-    if (b_podcastView_displayed) {
-        plrect.size.height -= [_podcastView frame].size.height;
-        plrect.origin.y = [_podcastView frame].size.height;
-    }
-    [_playlistScrollView setFrame:plrect];
-
-    [_dropzoneView setNeedsDisplay:YES];
-    [_playlistScrollView setNeedsDisplay:YES];
-}
-
+// Show split view and hide the video view
 - (void)makeSplitViewVisible
 {
     if (self.darkInterface)
@@ -457,13 +419,14 @@ static const float f_min_window_height = 307.;
     [self.videoView setHidden:YES];
     [_splitView setHidden:NO];
     if (self.nativeFullscreenMode && [self fullscreen]) {
-        [[self.controlsBar bottomBarView] setHidden:NO];
+        [self showControlsBar];
         [self.fspanel setNonActive];
     }
 
     [self makeFirstResponder:_playlistScrollView];
 }
 
+// Hides the split view and makes the vout view in foreground
 - (void)makeSplitViewHidden
 {
     if (self.darkInterface)
@@ -474,7 +437,7 @@ static const float f_min_window_height = 307.;
     [_splitView setHidden:YES];
     [self.videoView setHidden:NO];
     if (self.nativeFullscreenMode && [self fullscreen]) {
-        [[self.controlsBar bottomBarView] setHidden:YES];
+        [self hideControlsBar];
         [self.fspanel setActive];
     }
 
@@ -541,6 +504,7 @@ static const float f_min_window_height = 307.;
             [_splitView setHidden: NO];
             [_playlistScrollView setHidden: NO];
             [self.videoView setHidden: YES];
+            [self showControlsBar];
         }
     }
 
@@ -608,9 +572,6 @@ static const float f_min_window_height = 307.;
 
 - (void)hideSplitView:(BOOL)resize
 {
-    // cancel pending pl resizes, in case of fast toggle between both modes
-    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(resizePlaylistAfterCollapse) object:nil];
-
     if (resize) {
         NSRect winrect = [self frame];
         f_lastSplitViewHeight = [_splitView frame].size.height;
@@ -647,10 +608,6 @@ static const float f_min_window_height = 307.;
         [self setFrame:winrect display:YES animate:YES];
     }
 
-    // cancel pending pl resizes, in case of fast toggle between both modes
-    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(resizePlaylistAfterCollapse) object:nil];
-    [self performSelector:@selector(resizePlaylistAfterCollapse) withObject: nil afterDelay:0.75];
-
     b_splitview_removed = NO;
 }
 
@@ -829,42 +786,16 @@ static const float f_min_window_height = 307.;
     }
 
     if (self.nativeFullscreenMode) {
-        if ([self hasActiveVideo] && [self fullscreen]) {
-            [[self.controlsBar bottomBarView] setHidden: b_videoPlayback];
+        if ([self hasActiveVideo] && [self fullscreen] && b_videoPlayback) {
+            [self hideControlsBar];
             [self.fspanel setActive];
         } else {
-            [[self.controlsBar bottomBarView] setHidden: NO];
+            [self showControlsBar];
             [self.fspanel setNonActive];
         }
     }
 }
 
-#pragma mark -
-#pragma mark Lion native fullscreen handling
-- (void)windowWillEnterFullScreen:(NSNotification *)notification
-{
-    [super windowWillEnterFullScreen:notification];
-
-    // update split view frame after removing title bar
-    if (self.darkInterface) {
-        NSRect frame = [[self contentView] frame];
-        frame.origin.y += [self.controlsBar height];
-        frame.size.height -= [self.controlsBar height];
-        [_splitView setFrame:frame];
-    }
-}
-
-- (void)windowWillExitFullScreen:(NSNotification *)notification
-{
-    [super windowWillExitFullScreen: notification];
-
-    // update split view frame after readding title bar
-    if (self.darkInterface) {
-        NSRect frame = [_splitView frame];
-        frame.size.height -= [self.titlebarView frame].size.height;
-        [_splitView setFrame:frame];
-    }
-}
 #pragma mark -
 #pragma mark Fullscreen support
 
