@@ -2,7 +2,7 @@
  * VLCRendererDiscovery.m: Wrapper class for vlc_renderer_discovery_t
  *****************************************************************************
  * Copyright (C) 2016 VLC authors and VideoLAN
- * $Id: 2a4cfba03a1f2256b263e35be832e2925bd5b9c8 $
+ * $Id: 8270adc002440cdd1e1ffedab5eb852363a95aa7 $
  *
  * Authors: Marvin Scholz <epirat07 at gmail dot com>
  *
@@ -65,6 +65,7 @@ static void renderer_event_item_removed(vlc_renderer_discovery_t *rd,
                         format:@"name must not be nil"];
         _name = [NSString stringWithUTF8String:name];
         _longName = (!longname) ? nil : [NSString stringWithUTF8String:longname];
+        _rendererItems = [NSMutableArray array];
     }
     return self;
 }
@@ -119,13 +120,13 @@ static void renderer_event_item_removed(vlc_renderer_discovery_t *rd,
     for (VLCRendererItem *item in _rendererItems) {
         if (item.rendererItem == base_item) {
             result_item = item;
-            return;
+            break;
         }
     }
     if (result_item) {
-        [_rendererItems removeObject:result_item];
         if (_delegate)
             [_delegate removedRendererItem:result_item from:self];
+        [_rendererItems removeObject:result_item];
     } else {
         msg_Err(p_intf, "VLCRendererDiscovery could not find item to remove!");
     }
