@@ -2,7 +2,7 @@
  * VLCVideoEffectsWindowController.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2011-2015 Felix Paul Kühne
- * $Id: af2b337c3c8d59dc2ea0ac61b4c45ac8a65a58c4 $
+ * $Id: 79f5f1d2a325e5f24eb30f0f891f67922d52dbe3 $
  *
  * Authors: Felix Paul Kühne <fkuehne -at- videolan -dot- org>
  *
@@ -670,12 +670,21 @@
 - (void)addProfile:(id)sender
 {
     /* show panel */
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wpartial-availability"
+#ifdef MAC_OS_X_VERSION_10_10
+    if (OSX_YOSEMITE_AND_HIGHER) {
+        [[_textfieldPanel window] setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantDark]];
+    }
+#endif
+#pragma clang diagnostic pop
     [_textfieldPanel setTitleString:_NS("Duplicate current profile for a new profile")];
     [_textfieldPanel setSubTitleString:_NS("Enter a name for the new profile:")];
     [_textfieldPanel setCancelButtonString:_NS("Cancel")];
     [_textfieldPanel setOkButtonString:_NS("Save")];
 
-    __weak typeof(self) _self = self;
+    // TODO: Change to weak, when dropping 10.7 support
+    __unsafe_unretained typeof(self) _self = self;
     [_textfieldPanel runModalForWindow:self.window completionHandler:^(NSInteger returnCode, NSString *resultingText) {
 
         NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
@@ -728,13 +737,22 @@
 - (void)removeProfile:(id)sender
 {
     /* show panel */
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wpartial-availability"
+#ifdef MAC_OS_X_VERSION_10_10
+    if (OSX_YOSEMITE_AND_HIGHER) {
+        [[_popupPanel window] setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantDark]];
+    }
+#endif
+#pragma clang diagnostic pop
     [_popupPanel setTitleString:_NS("Remove a preset")];
     [_popupPanel setSubTitleString:_NS("Select the preset you would like to remove:")];
     [_popupPanel setOkButtonString:_NS("Remove")];
     [_popupPanel setCancelButtonString:_NS("Cancel")];
     [_popupPanel setPopupButtonContent:[[NSUserDefaults standardUserDefaults] objectForKey:@"VideoEffectProfileNames"]];
 
-    __weak typeof(self) _self = self;
+    // TODO: Change to weak, when dropping 10.7 support
+    __unsafe_unretained typeof(self) _self = self;
     [_popupPanel runModalForWindow:self.window completionHandler:^(NSInteger returnCode, NSInteger selectedIndex) {
 
         NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
