@@ -2,7 +2,7 @@
  * extended_panels.cpp : Extended controls panels
  ****************************************************************************
  * Copyright (C) 2006-2013 the VideoLAN team
- * $Id: d8c0f514ac332f41885d16a8ae16ea6085096910 $
+ * $Id: b1fab5d5babbb5fe92d25a5a1fab0f7b8d65a13e $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Antoine Cellerier <dionoea .t videolan d@t org>
@@ -53,9 +53,40 @@
 #include <vlc_modules.h>
 #include <vlc_plugin.h>
 
-static QString ChangeFiltersString( struct intf_thread_t *p_intf, const char *psz_filter_type, const char *psz_name, bool b_add );
-static void ChangeAFiltersString( struct intf_thread_t *p_intf, const char *psz_name, bool b_add );
-static void ChangeVFiltersString( struct intf_thread_t *p_intf, const char *psz_name, bool b_add );
+static bool filterIsPresent( const QString &filters, const QString &filter )
+{
+    QStringList list = filters.split( ':', QString::SplitBehavior::SkipEmptyParts );
+    foreach( const QString &filterCmp, list )
+    {
+        if( filterCmp.compare( filter ) == 0 )
+            return true;
+    }
+    return false;
+}
+
+static const char* GetVFilterType( struct intf_thread_t *p_intf, const char *psz_name )
+{
+    module_t *p_obj = module_find( psz_name );
+    if( !p_obj )
+    {
+        msg_Err( p_intf, "Unable to find filter module \"%s\".", psz_name );
+        return NULL;
+    }
+
+    if( module_provides( p_obj, "video splitter" ) )
+        return "video-splitter";
+    else if( module_provides( p_obj, "video filter" ) )
+        return "video-filter";
+    else if( module_provides( p_obj, "sub source" ) )
+        return "sub-source";
+    else if( module_provides( p_obj, "sub filter" ) )
+        return "sub-filter";
+    else
+    {
+        msg_Err( p_intf, "Unknown video filter type." );
+        return NULL;
+    }
+}
 
 static const QString ModuleFromWidgetName( QObject *obj )
 {
@@ -73,13 +104,18 @@ static QString OptionFromWidgetName( QObject *obj )
 
 static inline void setup_vfilter( intf_thread_t *p_intf, const char* psz_name, QWidget *widget )
 {
-    vlc_object_t *p_obj = ( vlc_object_t * )
-        vlc_object_find_name( p_intf->obj.libvlc, psz_name );
+    const char *psz_filter_type = GetVFilterType( p_intf, psz_name );
+    if( psz_filter_type == NULL )
+        return;
+
+    char *psz_filters = var_InheritString( THEPL, psz_filter_type );
+    if( psz_filters == NULL )
+        return;
+
     QCheckBox *checkbox = qobject_cast<QCheckBox*>( widget );
     QGroupBox *groupbox = qobject_cast<QGroupBox*>( widget );
-    if( p_obj )
+    if( filterIsPresent( qfu(psz_filters), qfu(psz_name) ) )
     {
-        vlc_object_release( p_obj ); \
         if( checkbox ) checkbox->setChecked( true ); \
         else if (groupbox) groupbox->setChecked( true ); \
     }
@@ -88,6 +124,7 @@ static inline void setup_vfilter( intf_thread_t *p_intf, const char* psz_name, Q
         if( checkbox ) checkbox->setChecked( false );
         else if (groupbox) groupbox->setChecked( false );
     }
+    free( psz_filters );
 }
 
 #define SETUP_VFILTER( widget ) \
@@ -157,7 +194,6 @@ ExtVideo::ExtVideo( intf_thread_t *_p_intf, QTabWidget *_parent ) :
     SETUP_VFILTER( puzzle )
     SETUP_VFILTER_OPTION( puzzleRowsSpin, valueChanged( int ) )
     SETUP_VFILTER_OPTION( puzzleColsSpin, valueChanged( int ) )
-    SETUP_VFILTER_OPTION( puzzleBlackSlotCheck, stateChanged( int ) )
 
     SETUP_VFILTER( magnify )
 
@@ -231,8 +267,8 @@ void ExtVideo::cropChange()
     if( ui.leftRightCropSync->isChecked() )
         ui.cropRightPx->setValue( ui.cropLeftPx->value() );
 
-    vout_thread_t *p_vout = THEMIM->getVout();
-    if( p_vout )
+    QVector<vout_thread_t*> p_vouts = THEMIM->getVouts();
+    foreach( vout_thread_t *p_vout, p_vouts )
     {
         var_SetInteger( p_vout, "crop-top", ui.cropTopPx->value() );
         var_SetInteger( p_vout, "crop-bottom", ui.cropBotPx->value() );
@@ -252,7 +288,7 @@ void ExtVideo::clean()
 
 static QString ChangeFiltersString( struct intf_thread_t *p_intf, const char *psz_filter_type, const char *psz_name, bool b_add )
 {
-    char* psz_chain = config_GetPsz( p_intf, psz_filter_type );
+    char* psz_chain = var_GetString( THEPL, psz_filter_type );
 
     QString const chain = QString( psz_chain ? psz_chain : "" );
     QStringList list = chain.split( ':', QString::SplitBehavior::SkipEmptyParts );
@@ -265,70 +301,34 @@ static QString ChangeFiltersString( struct intf_thread_t *p_intf, const char *ps
     return list.join( ":" );
 }
 
-static void ChangeAFiltersString( struct intf_thread_t *p_intf, const char *psz_name, bool b_add )
+static void UpdateVFiltersString( struct intf_thread_t *p_intf,
+                                  const char *psz_filter_type, const char *value )
 {
-    module_t *p_obj = module_find( psz_name );
-    if( !p_obj )
-    {
-        msg_Err( p_intf, "Unable to find filter module \"%s\".", psz_name );
-        return;
-    }
+    var_SetString( THEPL, psz_filter_type, value );
 
-    QString result = ChangeFiltersString( p_intf, "audio-filter", psz_name, b_add );
-    config_PutPsz( p_intf, "audio-filter", qtu( result ) );
-}
-
-static const char* GetVFilterType( struct intf_thread_t *p_intf, const char *psz_name )
-{
-    module_t *p_obj = module_find( psz_name );
-    if( !p_obj )
+    /* Try to set non splitter filters on the fly */
+    if( strcmp( psz_filter_type, "video-splitter" ) )
     {
-        msg_Err( p_intf, "Unable to find filter module \"%s\".", psz_name );
-        return NULL;
-    }
-
-    if( module_provides( p_obj, "video splitter" ) )
-        return "video-splitter";
-    else if( module_provides( p_obj, "video filter" ) )
-        return "video-filter";
-    else if( module_provides( p_obj, "sub source" ) )
-        return "sub-source";
-    else if( module_provides( p_obj, "sub filter" ) )
-        return "sub-filter";
-    else
-    {
-        msg_Err( p_intf, "Unknown video filter type." );
-        return NULL;
+        QVector<vout_thread_t*> p_vouts = THEMIM->getVouts();
+        foreach( vout_thread_t *p_vout, p_vouts )
+        {
+            var_SetString( p_vout, psz_filter_type, value );
+            vlc_object_release( p_vout );
+        }
     }
 }
 
-static void ChangeVFiltersString( struct intf_thread_t *p_intf, const char *psz_name, bool b_add )
+void ExtVideo::changeVFiltersString( const char *psz_name, bool b_add )
 {
     const char *psz_filter_type = GetVFilterType( p_intf, psz_name );
-
     if( psz_filter_type == NULL )
         return;
 
     QString result = ChangeFiltersString( p_intf, psz_filter_type, psz_name, b_add );
 
-    /* Vout is not kept, so put that in the config */
-    config_PutPsz( p_intf, psz_filter_type, qtu( result ) );
+    emit configChanged( qfu( psz_filter_type ), result );
 
-    /* Try to set on the fly */
-    if( !strcmp( psz_filter_type, "video-splitter" ) )
-    {
-        playlist_t *p_playlist = THEPL;
-        var_SetString( p_playlist, psz_filter_type, qtu( result ) );
-    }
-    else
-    {
-        vout_thread_t *p_vout = THEMIM->getVout();
-        if( p_vout )
-        {
-            var_SetString( p_vout, psz_filter_type, qtu( result ) );
-            vlc_object_release( p_vout );
-        }
-    }
+    UpdateVFiltersString( p_intf, psz_filter_type, qtu( result ) );
 }
 
 void ExtVideo::updateFilters()
@@ -338,7 +338,7 @@ void ExtVideo::updateFilters()
     QCheckBox *checkbox = qobject_cast<QCheckBox*>( sender() );
     QGroupBox *groupbox = qobject_cast<QGroupBox*>( sender() );
 
-    ChangeVFiltersString( p_intf, qtu( module ),
+    changeVFiltersString( qtu( module ),
                           checkbox ? checkbox->isChecked()
                                    : groupbox->isChecked() );
 }
@@ -420,39 +420,25 @@ void ExtVideo::setWidgetValue( QObject *widget )
     QString option = OptionFromWidgetName( widget );
     //std::cout << "Option name: " << option.toStdString() << std::endl;
 
-    vlc_object_t *p_obj = ( vlc_object_t * )
-        vlc_object_find_name( p_intf->obj.libvlc, qtu( module ) );
-    int i_type;
     vlc_value_t val;
-
-    if( !p_obj )
+    int i_type = config_GetType( p_intf, qtu( option ) ) & VLC_VAR_CLASS;
+    switch( i_type )
     {
-#if 0
-        msg_Dbg( p_intf,
-                 "Module instance %s not found, looking in config values.",
-                 qtu( module ) );
-#endif
-        i_type = config_GetType( p_intf, qtu( option ) ) & VLC_VAR_CLASS;
-        switch( i_type )
-        {
-            case VLC_VAR_INTEGER:
-            case VLC_VAR_BOOL:
-                val.i_int = config_GetInt( p_intf, qtu( option ) );
-                break;
-            case VLC_VAR_FLOAT:
-                val.f_float = config_GetFloat( p_intf, qtu( option ) );
-                break;
-            case VLC_VAR_STRING:
-                val.psz_string = config_GetPsz( p_intf, qtu( option ) );
-                break;
-        }
-    }
-    else
-    {
-        i_type = var_Type( p_obj, qtu( option ) ) & VLC_VAR_CLASS;
-        var_Get( p_obj, qtu( option ), &val );
-        vlc_object_release( p_obj );
+        case VLC_VAR_INTEGER:
+        case VLC_VAR_BOOL:
+        case VLC_VAR_FLOAT:
+        case VLC_VAR_STRING:
+            break;
+        default:
+            msg_Err( p_intf,
+                     "Module %s's %s variable is of an unsupported type ( %d )",
+                     qtu( module ), qtu( option ), i_type );
+            return;
     }
+    if( var_Create( THEPL, qtu( option ), i_type | VLC_VAR_DOINHERIT ) )
+        return;
+    if( var_GetChecked( THEPL, qtu( option ), i_type, &val ) )
+        return;
 
     /* Try to cast to all the widgets we're likely to encounter. Only
      * one of the casts is expected to work. */
@@ -496,83 +482,86 @@ void ExtVideo::setWidgetValue( QObject *widget )
         else msg_Warn( p_intf, "Could not find the correct String widget" );
         free( val.psz_string );
     }
-    else
-        if( p_obj )
-            msg_Err( p_intf,
-                     "Module %s's %s variable is of an unsupported type ( %d )",
-                     qtu( module ),
-                     qtu( option ),
-                     i_type );
 }
 
-void ExtVideo::setFilterOption( struct intf_thread_t *p_intf, const char *psz_module, const char *psz_option,
-        int i_int, double f_float, QString val )
+void ExtVideo::setFilterOption( const char *psz_module, const char *psz_option,
+        int i_int, double f_float, const char *psz_string )
 {
-    vlc_object_t *p_obj = ( vlc_object_t * )vlc_object_find_name( p_intf->obj.libvlc, psz_module );
-    int i_type;
-    bool b_is_command;
+    QVector<vout_thread_t*> p_vouts = THEMIM->getVouts();
+    int i_type = 0;
+    bool b_is_command = false;
 
-    if( !p_obj )
+    if( !p_vouts.isEmpty() )
     {
-        msg_Warn( p_intf, "Module %s not found. You'll need to restart the filter to take the change into account.", psz_module );
-        i_type = config_GetType( p_intf, psz_option );
-        b_is_command = false;
-    }
-    else
-    {
-        i_type = var_Type( p_obj, psz_option );
-        if( i_type == 0 )
-            i_type = config_GetType( p_intf, psz_option );
+        i_type = var_Type( p_vouts.at(0), psz_option );
         b_is_command = ( i_type & VLC_VAR_ISCOMMAND );
     }
+    if( i_type == 0 )
+        i_type = config_GetType( p_intf, psz_option );
 
+    vlc_value_t val;
     i_type &= VLC_VAR_CLASS;
     if( i_type == VLC_VAR_INTEGER || i_type == VLC_VAR_BOOL )
     {
-        if( i_int == -1 )
-            msg_Warn( p_intf, "Could not find the correct Integer widget" );
-        config_PutInt( p_intf, psz_option, i_int );
-        if( b_is_command )
+        emit configChanged( qfu( psz_option ), QVariant( i_int ) );
+        if( i_type == VLC_VAR_INTEGER )
         {
-            if( i_type == VLC_VAR_INTEGER )
-                var_SetInteger( p_obj, psz_option, i_int );
-            else
-                var_SetBool( p_obj, psz_option, i_int );
+            val.i_int = i_int;
+            var_SetInteger( THEPL, psz_option, i_int );
+        }
+        else
+        {
+            var_SetBool( THEPL, psz_option, i_int );
+            val.b_bool = i_int;
         }
     }
     else if( i_type == VLC_VAR_FLOAT )
     {
-        if( f_float == -1 )
-            msg_Warn( p_intf, "Could not find the correct Float widget" );
-        config_PutFloat( p_intf, psz_option, f_float );
-        if( b_is_command )
-            var_SetFloat( p_obj, psz_option, f_float );
+        emit configChanged( qfu( psz_option ), QVariant( f_float ) );
+        var_SetFloat( THEPL, psz_option, f_float );
+        val.f_float = f_float;
     }
     else if( i_type == VLC_VAR_STRING )
     {
-        if( val.isNull() )
-            msg_Warn( p_intf, "Could not find the correct String widget" );
-        config_PutPsz( p_intf, psz_option, qtu( val ) );
-        if( b_is_command )
-            var_SetString( p_obj, psz_option, qtu( val ) );
+        if( psz_string == NULL )
+            psz_string = "";
+        emit configChanged( qfu( psz_option ), QVariant( psz_string ) );
+        var_SetString( THEPL, psz_option, psz_string );
+        val.psz_string = (char *) psz_string;
     }
     else
+    {
         msg_Err( p_intf,
                  "Module %s's %s variable is of an unsupported type ( %d )",
                  psz_module,
                  psz_option,
                  i_type );
+        b_is_command = false;
+    }
 
-    if( !b_is_command )
+    if( b_is_command )
+    {
+        foreach( vout_thread_t *p_vout, p_vouts )
+        {
+            var_SetChecked( p_vout, psz_option, i_type, val );
+#ifndef NDEBUG
+            int i_cur_type = var_Type( p_vout, psz_option );
+            assert( ( i_cur_type & VLC_VAR_CLASS ) == i_type );
+            assert( !!( i_cur_type & VLC_VAR_ISCOMMAND ) == b_is_command );
+#endif
+        }
+    }
+    else if( !p_vouts.isEmpty() )
     {
         msg_Warn( p_intf, "Module %s's %s variable isn't a command. Brute-restarting the filter.",
                  psz_module,
                  psz_option );
-        ChangeVFiltersString( p_intf, psz_module, false );
-        ChangeVFiltersString( p_intf, psz_module, true );
+        changeVFiltersString( psz_module, false );
+        changeVFiltersString( psz_module, true );
     }
 
-    if( p_obj ) vlc_object_release( p_obj );
+    foreach( vout_thread_t *p_vout, p_vouts )
+        vlc_object_release( p_vout );
 }
 
 void ExtVideo::updateFilterOptions()
@@ -617,7 +606,7 @@ void ExtVideo::updateFilterOptions()
         val = combobox->itemData( combobox->currentIndex() ).toString();
     }
 
-    setFilterOption( p_intf, qtu( module ), qtu( option ), i_int, f_float, val);
+    setFilterOption( qtu( module ), qtu( option ), i_int, f_float, qtu( val ) );
 }
 
 /**********************************************************************
@@ -835,7 +824,6 @@ void ExtV4l2::ValueChange( int value )
 FilterSliderData::FilterSliderData( QObject *parent, QSlider *_slider ) :
     QObject( parent ), slider( _slider )
 {
-    b_save_to_config = false;
 }
 
 FilterSliderData::FilterSliderData( QObject *parent,
@@ -846,7 +834,6 @@ FilterSliderData::FilterSliderData( QObject *parent,
     QObject( parent ), slider( _slider ), valueLabel( _label ),
     nameLabel( _nameLabel ), p_data( _p_data ), p_intf( _p_intf )
 {
-    b_save_to_config = false;
     slider->setMinimum( p_data->f_min / p_data->f_resolution );
     slider->setMaximum( p_data->f_max / p_data->f_resolution );
     nameLabel->setText( p_data->descs );
@@ -898,7 +885,7 @@ float FilterSliderData::initialValue()
     return f;
 }
 
-void FilterSliderData::onValueChanged( int i ) const
+void FilterSliderData::onValueChanged( int i )
 {
     float f = ((float) i) * p_data->f_resolution;
     vlc_object_t *p_aout = (vlc_object_t *) THEMIM->getAout();
@@ -910,16 +897,10 @@ void FilterSliderData::onValueChanged( int i ) const
     writeToConfig();
 }
 
-void FilterSliderData::writeToConfig() const
+void FilterSliderData::writeToConfig()
 {
-    if ( !b_save_to_config ) return;
     float f = ((float) slider->value()) * p_data->f_resolution;
-    config_PutFloat( p_intf, qtu(p_data->name), f );
-}
-
-void FilterSliderData::setSaveToConfig( bool b )
-{
-    b_save_to_config = b;
+    emit configChanged( p_data->name, QVariant( f ) );
 }
 
 AudioFilterControlWidget::AudioFilterControlWidget
@@ -927,6 +908,12 @@ AudioFilterControlWidget::AudioFilterControlWidget
     QWidget( parent ), p_intf( _p_intf ), name( _name ), i_smallfont(0)
 {}
 
+void AudioFilterControlWidget::connectConfigChanged( FilterSliderData *slider )
+{
+    connect( slider, SIGNAL( configChanged(QString, QVariant) ),
+             this, SIGNAL( configChanged(QString, QVariant) ) );
+}
+
 void AudioFilterControlWidget::build()
 {
     QFont smallFont = QApplication::font();
@@ -957,19 +944,12 @@ void AudioFilterControlWidget::build()
         ctrlLayout->addWidget( nameLabel, 2, i, Qt::AlignHCenter );
         i++;
         sliderDatas << filter;
+        connectConfigChanged( filter );
     }
 
-    vlc_object_t *p_aout = (vlc_object_t *)THEMIM->getAout();
-    char *psz_af;
-    if( p_aout )
-    {
-        psz_af = var_GetNonEmptyString( p_aout, "audio-filter" );
-        vlc_object_release( p_aout );
-    }
-    else
-        psz_af = config_GetPsz( p_intf, "audio-filter" );
+    char *psz_af = var_InheritString( THEPL, "audio-filter" );
 
-    if( psz_af && strstr( psz_af, qtu(name) ) != NULL )
+    if( psz_af && filterIsPresent( qfu(psz_af), name ) )
         slidersBox->setChecked( true );
     else
         slidersBox->setChecked( false );
@@ -978,16 +958,19 @@ void AudioFilterControlWidget::build()
     free( psz_af );
 }
 
-void AudioFilterControlWidget::enable( bool b_enable ) const
+void AudioFilterControlWidget::enable( bool b_enable )
 {
-    ChangeAFiltersString( p_intf, qtu(name), b_enable );
-    playlist_EnableAudioFilter( THEPL, qtu(name), b_enable );
-}
+    module_t *p_obj = module_find( qtu(name) );
+    if( !p_obj )
+    {
+        msg_Err( p_intf, "Unable to find filter module \"%s\".", qtu(name) );
+        return;
+    }
 
-void AudioFilterControlWidget::setSaveToConfig( bool b_save )
-{
-    foreach( FilterSliderData *f, sliderDatas )
-        f->setSaveToConfig( b_save );
+    QString result = ChangeFiltersString( p_intf, "audio-filter", qtu(name),
+                                          b_enable );
+    emit configChanged( qfu("audio-filter"), result );
+    playlist_EnableAudioFilter( THEPL, qtu(name), b_enable );
 }
 
 /**********************************************************************
@@ -1059,7 +1042,7 @@ float EqualizerSliderData::initialValue()
     return f;
 }
 
-void EqualizerSliderData::onValueChanged( int i ) const
+void EqualizerSliderData::onValueChanged( int i )
 {
     QStringList bands = getBandsFromAout();
     if ( bands.count() > index )
@@ -1076,15 +1059,14 @@ void EqualizerSliderData::onValueChanged( int i ) const
     }
 }
 
-void EqualizerSliderData::writeToConfig() const
+void EqualizerSliderData::writeToConfig()
 {
-    if ( !b_save_to_config ) return;
     QStringList bands = getBandsFromAout();
     if ( bands.count() > index )
     {
         float f = (float) slider->value() * p_data->f_resolution;
         bands[ index ] = QLocale( QLocale::C ).toString( f );
-        config_PutPsz( p_intf, qtu(p_data->name), qtu(bands.join( " " )) );
+        emit configChanged( p_data->name, QVariant( bands.join( " " ) ) );
     }
 }
 
@@ -1143,6 +1125,7 @@ void Equalizer::build()
     ui.preampValue->setFont( smallFont );
     preamp = new FilterSliderData( this, p_intf,
         ui.preampSlider, ui.preampValue, ui.preampLabel, & preamp_values );
+    connectConfigChanged( preamp );
 
     /* fix sliders spacing accurately */
     int i_width = qMax( QFontMetrics( smallFont ).width( "500 Hz" ),
@@ -1166,6 +1149,7 @@ void Equalizer::build()
         ctrlLayout->addWidget( nameLabel, 1, i, Qt::AlignHCenter );
         sliderDatas << filter; /* keep track for applying presets */
         i++;
+        connectConfigChanged( filter );
     }
 
     /* Add the listed presets */
@@ -1197,12 +1181,9 @@ void Equalizer::build()
     vlc_object_t *p_aout = (vlc_object_t *)THEMIM->getAout();
     char *psz_af;
     if( p_aout )
-    {
         psz_af = var_GetNonEmptyString( p_aout, "audio-filter" );
-        vlc_object_release( p_aout );
-    }
     else
-        psz_af = config_GetPsz( p_intf, "audio-filter" );
+        psz_af = var_InheritString( THEPL, "audio-filter" );
 
     /* To enable or disable subwidgets */
     /* If that list grows, better iterate over layout's childs */
@@ -1214,7 +1195,7 @@ void Equalizer::build()
     CONNECT( ui.enableCheck, toggled(bool), ui.preampValue, setEnabled(bool) );
     CONNECT( ui.enableCheck, toggled(bool), ui.preampLabel, setEnabled(bool) );
 
-    if( psz_af && strstr( psz_af, qtu(name) ) != NULL )
+    if( psz_af && filterIsPresent( qfu(psz_af), name ) )
         ui.enableCheck->setChecked( true );
     else
         ui.enableCheck->setChecked( false );
@@ -1228,12 +1209,8 @@ void Equalizer::build()
     /* Connect and set 2 Pass checkbox */
     ui.eq2PassCheck->setChecked( var_InheritBool( p_aout, "equalizer-2pass" ) );
     CONNECT( ui.eq2PassCheck, toggled(bool), this, enable2Pass(bool) );
-}
-
-void Equalizer::setSaveToConfig( bool b_save )
-{
-    AudioFilterControlWidget::setSaveToConfig( b_save );
-    preamp->setSaveToConfig( b_save );
+    if( p_aout )
+        vlc_object_release( p_aout );
 }
 
 void Equalizer::setCorePreset( int i_preset )
@@ -1254,11 +1231,11 @@ void Equalizer::setCorePreset( int i_preset )
         var_SetString( p_aout , "equalizer-preset" , preset_list[i_preset] );
         vlc_object_release( p_aout );
     }
-    config_PutPsz( p_intf, "equalizer-preset", preset_list[i_preset] );
+    emit configChanged( qfu( "equalizer-preset" ), QVariant( qfu( preset_list[i_preset] ) ) );
 }
 
 /* Function called when the set2Pass button is activated */
-void Equalizer::enable2Pass( bool b_enable ) const
+void Equalizer::enable2Pass( bool b_enable )
 {
     vlc_object_t *p_aout= (vlc_object_t *)THEMIM->getAout();
 
@@ -1267,7 +1244,7 @@ void Equalizer::enable2Pass( bool b_enable ) const
         var_SetBool( p_aout, "equalizer-2pass", b_enable );
         vlc_object_release( p_aout );
     }
-    config_PutInt( p_intf, "equalizer-2pass", b_enable );
+    emit configChanged( qfu( "equalizer-2pass" ), QVariant( b_enable ) );
 }
 
 /**********************************************************************
@@ -1524,7 +1501,7 @@ void SyncControls::adjustSubsDuration( double f_factor )
     if( THEMIM->getInput() && b_userAction )
     {
         subsdelaySetFactor( f_factor );
-        ChangeVFiltersString( p_intf, "subsdelay", f_factor > 0 );
+        changeVFiltersString( "subsdelay", f_factor > 0 );
     }
 }
 
@@ -1557,14 +1534,11 @@ void SyncControls::initSubsDuration()
 void SyncControls::subsdelayClean()
 {
     /* Remove subsdelay filter */
-    ChangeVFiltersString( p_intf, "subsdelay", false );
+    changeVFiltersString( "subsdelay", false );
 }
 
 void SyncControls::subsdelaySetFactor( double f_factor )
 {
-    /* Set the factor in the preferences */
-    config_PutFloat( p_intf, SUBSDELAY_CFG_FACTOR, f_factor );
-
     /* Try to find an instance of subsdelay, and set its factor */
     vlc_object_t *p_obj = ( vlc_object_t * ) vlc_object_find_name( p_intf->obj.libvlc, "subsdelay" );
     if( p_obj )
@@ -1574,6 +1548,17 @@ void SyncControls::subsdelaySetFactor( double f_factor )
     }
 }
 
+void SyncControls::changeVFiltersString( const char *psz_name, bool b_add )
+{
+    const char *psz_filter_type = GetVFilterType( p_intf, psz_name );
+    if( psz_filter_type == NULL )
+        return;
+
+    QString result = ChangeFiltersString( p_intf, psz_filter_type, psz_name, b_add );
+
+    UpdateVFiltersString( p_intf, psz_filter_type, qtu( result ) );
+}
+
 
 /**********************************************************************
  * Video filters / Adjust
