@@ -2,7 +2,7 @@
  * interface_widgets.cpp : Custom widgets for the main interface
  ****************************************************************************
  * Copyright (C) 2006-2010 the VideoLAN team
- * $Id: 7ca12a1efeaaf8f52b7dd7ff8a8ca4c4242eb344 $
+ * $Id: d483461cb5263db04e832e56fd29099542a2c250 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -202,7 +202,7 @@ QSize VideoWidget::physicalSize() const
     HWND hwnd;
     RECT rect;
 
-    QWindow *window = stable->windowHandle();
+    QWindow *window = windowHandle();
     hwnd = static_cast<HWND>(QGuiApplication::platformNativeInterface()->nativeResourceForWindow("handle", window));
 
     GetClientRect(hwnd, &rect);
