@@ -2,7 +2,7 @@
  * plugins.cpp : Plug-ins and extensions listing
  ****************************************************************************
  * Copyright (C) 2008-2010 the VideoLAN team
- * $Id: 4fc1d029668bb642b07a77b8b19bd03c0f7b6a5c $
+ * $Id: c1d7463a201d789b847d98882cbfec18b7050618 $
  *
  * Authors: Jean-Baptiste Kempf <jb (at) videolan.org>
  *          Jean-Philippe André <jpeg (at) videolan.org>
@@ -870,6 +870,11 @@ AddonsListModel::AddonsListModel( AddonsManager *AM_, QObject *parent )
 
 }
 
+AddonsListModel::~AddonsListModel()
+{
+    qDeleteAll( addons );
+}
+
 void AddonsListModel::addonAdded(  addon_entry_t *p_entry )
 {
     beginInsertRows( QModelIndex(), addons.count(), addons.count() );
@@ -903,9 +908,7 @@ Qt::ItemFlags AddonsListModel::flags( const QModelIndex &index ) const
     int i_state = data( index, StateRole ).toInt();
 
     if ( i_state == ADDON_UNINSTALLING || i_state == ADDON_INSTALLING )
-    {
-        i_flags &= !Qt::ItemIsEnabled;
-    }
+        i_flags &= ~Qt::ItemIsEnabled;
 
     i_flags |= Qt::ItemIsEditable;
 
