@@ -2,7 +2,7 @@
  * dialogs_provider.cpp : Dialog Provider
  *****************************************************************************
  * Copyright (C) 2006-2009 the VideoLAN team
- * $Id: e141bbc9dee4d0fea35cedde16000d2d3cf5d295 $
+ * $Id: 1e26f1f71e92613d37d7e0fa175725038594604f $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -620,7 +620,7 @@ void DialogsProvider::openAPlaylist()
     }
 }
 
-void DialogsProvider::saveAPlaylist(playlist_t *p_playlist, playlist_item_t *p_node)
+void DialogsProvider::savePlayingToPlaylist()
 {
     static const struct
     {
@@ -686,33 +686,12 @@ void DialogsProvider::saveAPlaylist(playlist_t *p_playlist, playlist_item_t *p_n
 
     if ( psz_selected_module )
     {
-        playlist_Export( p_playlist, qtu( toNativeSeparators( file ) ),
+        playlist_Export( THEPL, qtu( toNativeSeparators( file ) ),
                          true, psz_selected_module );
         getSettings()->setValue( "last-playlist-ext", psz_last_playlist_ext );
     }
 }
 
-void DialogsProvider::savePlayingToPlaylist()
-{
-    saveAPlaylist(THEPL, THEPL->p_playing);
-}
-
-void DialogsProvider::saveRecentsToPlaylist()
-{
-    playlist_item_t *p_node_recents = RecentsMRL::getInstance(p_intf)->toPlaylist(0);
-
-    if (p_node_recents == NULL)
-    {
-        msg_Warn(p_intf, "cannot create playlist from recents");
-        return;
-    }
-
-    saveAPlaylist(THEPL, p_node_recents);
-    playlist_Lock(THEPL);
-    playlist_NodeDelete(THEPL, p_node_recents, false);
-    playlist_Unlock(THEPL);
-}
-
 /****************************************************************************
  * Sout emulation
  ****************************************************************************/
