@@ -2,7 +2,7 @@
  * ts.c: MPEG-II TS Muxer
  *****************************************************************************
  * Copyright (C) 2001-2005 VLC authors and VideoLAN
- * $Id: 79bde06c40c30d241fc981dba923445a71cb5c18 $
+ * $Id: 8a2366ab854fa9bfbd94a4c897d411c13610bea7 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -333,8 +333,8 @@ typedef struct
 
 typedef struct
 {
-    ts_stream_t  ts;
-    pes_stream_t pes;
+    tsmux_stream_t  ts;
+    pesmux_stream_t pes;
     pes_state_t  state;
 } sout_input_sys_t;
 
@@ -354,10 +354,10 @@ struct sout_mux_sys_t
     unsigned        i_num_pmt;
     int             i_pmtslots;
     int             i_pat_version_number;
-    ts_stream_t     pat;
+    tsmux_stream_t  pat;
 
     int             i_pmt_version_number;
-    ts_stream_t     pmt[MAX_PMT];
+    tsmux_stream_t  pmt[MAX_PMT];
     pmt_map_t       pmtmap[MAX_PMT_PID];
     int             i_pmt_program_number[MAX_PMT];
     bool            b_data_alignment;
