@@ -2,7 +2,7 @@
  * hevc.c: h.265/hevc video packetizer
  *****************************************************************************
  * Copyright (C) 2014 VLC authors and VideoLAN
- * $Id: 7260fc59db004e2722d9568d82b8b63b9171279b $
+ * $Id: 39a85567b7dc9a4143ac2b8182609f82b0f7ae61 $
  *
  * Authors: Denis Charmet <typx@videolan.org>
  *
@@ -666,6 +666,8 @@ static block_t * ParseAUTail(decoder_t *p_dec, uint8_t i_nal_type, block_t *p_na
         case HEVC_NAL_EOS:
         case HEVC_NAL_EOB:
             p_ret = OutputQueues(p_sys, true);
+            if( p_ret )
+                p_ret->i_flags |= BLOCK_FLAG_END_OF_SEQUENCE;
             break;
 
         case HEVC_NAL_SUFF_SEI:
