@@ -2,7 +2,7 @@
  * copy.h: Fast YV12/NV12 copy
  *****************************************************************************
  * Copyright (C) 2009 Laurent Aimar
- * $Id: f32bd24c608cb5d7ca22f68faa14efe0f3250ffc $
+ * $Id: 9b9dbd1f66ec3d5a5ac213e53dfb0021d1f0ab24 $
  *
  * Authors: Laurent Aimar <fenrir_AT_ videolan _DOT_ org>
  *
@@ -53,4 +53,15 @@ void CopyFromI420ToNv12(picture_t *dst, uint8_t *src[3], size_t src_pitch[3],
 void CopyFromI420_10ToP010(picture_t *dst, uint8_t *src[3], size_t src_pitch[3],
                         unsigned height, copy_cache_t *cache);
 
+/**
+ * This functions sets the internal plane pointers/dimensions for the given
+ * buffer.
+ * This is useful when mapping opaque surfaces into CPU planes.
+ *
+ * picture is the picture to update
+ * data is the buffer pointer to use as the start of data for all the planes
+ * pitch is the internal line pitch for the buffer
+ */
+int picture_UpdatePlanes(picture_t *picture, uint8_t *data, unsigned pitch);
+
 #endif
