@@ -2,7 +2,7 @@
  * deinterlace.c : deinterlacer plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2011 VLC authors and VideoLAN
- * $Id: 5e56eb88e047edd4fe14c39994d97e93dc302f5d $
+ * $Id: c87d9d6b2c8517e0e74cff70d7474b97913bebf4 $
  *
  * Author: Sam Hocevar <sam@zoy.org>
  *         Christophe Massiot <massiot@via.ecp.fr>
@@ -148,15 +148,18 @@ static void SetFilterMethod( filter_t *p_filter, const char *mode, bool pack )
 {
     filter_sys_t *p_sys = p_filter->p_sys;
 
-    if( mode == NULL )
-        mode = "blend";
+    if ( mode == NULL )
+        mode = "auto";
 
-    p_sys->i_mode = DEINTERLACE_BLEND; /* default */
     p_sys->b_double_rate = false;
     p_sys->b_half_height = false;
     p_sys->b_use_frame_history = false;
 
-    if( !strcmp( mode, "discard" ) )
+    if ( !strcmp( mode, "auto" ) )
+    {
+        p_sys->i_mode = DEINTERLACE_X;
+    }
+    else if( !strcmp( mode, "discard" ) )
     {
         p_sys->i_mode = DEINTERLACE_DISCARD;
         p_sys->b_half_height = true;
@@ -178,12 +181,13 @@ static void SetFilterMethod( filter_t *p_filter, const char *mode, bool pack )
     }
     else if( !strcmp( mode, "blend" ) )
     {
+        p_sys->i_mode = DEINTERLACE_BLEND;
     }
     else if( pack )
     {
         msg_Err( p_filter, "unknown or incompatible deinterlace mode \"%s\""
                  " for packed format", mode );
-        mode = "blend";
+        return SetFilterMethod( p_filter, "auto", pack );
     }
     else if( !strcmp( mode, "yadif" ) )
     {
@@ -200,7 +204,7 @@ static void SetFilterMethod( filter_t *p_filter, const char *mode, bool pack )
     {
         msg_Err( p_filter, "unknown or incompatible deinterlace mode \"%s\""
                  " for high depth format", mode );
-        mode = "blend";
+        return SetFilterMethod(p_filter, "auto", pack);
     }
     else if( !strcmp( mode, "x" ) )
     {
@@ -609,7 +613,7 @@ int Open( vlc_object_t *p_this )
     if( chroma == NULL || chroma->pixel_size > 2 )
     {
 notsupp:
-        msg_Err( p_filter, "unsupported chroma %4.4s", (char*)&fourcc );
+        msg_Dbg( p_filter, "unsupported chroma %4.4s", (char*)&fourcc );
         return VLC_EGENERIC;
     }
 
