@@ -2,7 +2,7 @@
  * filters.c : audio output filters management
  *****************************************************************************
  * Copyright (C) 2002-2007 VLC authors and VideoLAN
- * $Id: 175a8b09f18a5f9bc9323e2b13ae871ad2c993f3 $
+ * $Id: 400d6e72f73cb15c076941ae3be27646e44cd93d $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -139,24 +139,6 @@ static int aout_FiltersPipelineCreate(vlc_object_t *obj, filter_t **filters,
     audio_sample_format_t input = *infmt;
     unsigned n = 0;
 
-    /* Encapsulate or decode non-linear formats */
-    if (!AOUT_FMT_LINEAR(infmt) && infmt->i_format != outfmt->i_format)
-    {
-        if (n == max)
-            goto overflow;
-
-        filter_t *f = TryFormat (obj, VLC_CODEC_S32N, &input);
-        if (f == NULL)
-            f = TryFormat (obj, VLC_CODEC_FL32, &input);
-        if (f == NULL)
-        {
-            msg_Err (obj, "cannot find %s for conversion pipeline",
-                     "decoder");
-            goto error;
-        }
-
-        filters[n++] = f;
-    }
     assert (AOUT_FMT_LINEAR(&input));
 
     /* Remix channels */
@@ -445,7 +427,6 @@ aout_filters_t *aout_FiltersNew (vlc_object_t *obj,
     if (request_vout != NULL)
         var_AddCallback (obj, "visual", VisualizationCallback, NULL);
 
-    /* Now add user filters */
     if (!AOUT_FMT_LINEAR(outfmt))
     {   /* Non-linear output: just convert formats, no filters/visu */
         if (!AOUT_FMTS_IDENTICAL(infmt, outfmt))
@@ -475,6 +456,7 @@ aout_filters_t *aout_FiltersNew (vlc_object_t *obj,
             filters->rate_filter = filters->tab[filters->count - 1];
     }
 
+    /* Now add user filters */
     char *str = var_InheritString (obj, "audio-filter");
     if (str != NULL)
     {
