@@ -2,7 +2,7 @@
  * control.c
  *****************************************************************************
  * Copyright (C) 1999-2015 VLC authors and VideoLAN
- * $Id: 6062a30a0d23113bd8d3b1ef0278b6e70218ab58 $
+ * $Id: 24fe630473c73313830438ba2f17038cf1ffb356 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -559,7 +559,7 @@ int input_vaControl( input_thread_t *p_input, int i_query, va_list args )
         }
 
         default:
-            msg_Err( p_input, "unknown query in input_vaControl" );
+            msg_Err( p_input, "unknown query 0x%x in %s", i_query, __func__ );
             return VLC_EGENERIC;
     }
 }
