@@ -2,7 +2,7 @@
  * es_out_timeshift.c: Es Out timeshift.
  *****************************************************************************
  * Copyright (C) 2008 Laurent Aimar
- * $Id: c4d18201817fb9a11d56cb3bff9d447543ede1e3 $
+ * $Id: c8fa798561a5ad68f90d6ff48f61d98ee4aac18b $
  *
  * Authors: Laurent Aimar < fenrir _AT_ videolan _DOT_ org>
  *
@@ -717,15 +717,15 @@ static int ControlLocked( es_out_t *p_out, int i_query, va_list args )
     {
         return ControlLockedSetFrameNext( p_out );
     }
+
     case ES_OUT_GET_PCR_SYSTEM:
-    {
         if( p_sys->b_delayed )
             return VLC_EGENERIC;
+        /* fall through */
+    case ES_OUT_GET_GROUP_FORCED:
+    case ES_OUT_POST_SUBNODE:
+        return es_out_vaControl( p_sys->p_out, i_query, args );
 
-        mtime_t *pi_system = (mtime_t*)va_arg( args, mtime_t * );
-        mtime_t *pi_delay  = (mtime_t*)va_arg( args, mtime_t * );
-        return es_out_ControlGetPcrSystem( p_sys->p_out, pi_system, pi_delay );
-    }
     case ES_OUT_MODIFY_PCR_SYSTEM:
     {
         const bool    b_absolute = va_arg( args, int );
@@ -736,11 +736,6 @@ static int ControlLocked( es_out_t *p_out, int i_query, va_list args )
 
         return es_out_ControlModifyPcrSystem( p_sys->p_out, b_absolute, i_system );
     }
-    case ES_OUT_GET_GROUP_FORCED:
-    {
-        int *pi_group = va_arg( args, int * );
-        return es_out_Control( p_sys->p_out, ES_OUT_GET_GROUP_FORCED, pi_group );
-    }
 
     default:
         msg_Err( p_sys->p_input, "Unknown es_out_Control query !" );
