@@ -2,7 +2,7 @@
  * item.c: input_item management
  *****************************************************************************
  * Copyright (C) 1998-2004 VLC authors and VideoLAN
- * $Id: 299927fe3b17e4b25141f4f91313d3685d312db8 $
+ * $Id: a8deb19b7053f3f8c2a1c729385ec44daf06f5d0 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -35,6 +35,7 @@
 
 #include "item.h"
 #include "info.h"
+#include "input_internal.h"
 
 struct input_item_opaque
 {
@@ -226,17 +227,6 @@ void input_item_CopyOptions( input_item_t *p_child,
     free( optv );
 }
 
-/* This won't hold the item, but can tell to interested third parties
- * Like the playlist, that there is a new sub item. With this design
- * It is not the input item's responsibility to keep all the ref of
- * the input item children. */
-void input_item_PostSubItem( input_item_t *p_parent, input_item_t *p_child )
-{
-    input_item_node_t *p_node = input_item_node_Create( p_parent );
-    input_item_node_AppendItem( p_node, p_child );
-    input_item_node_PostAndDelete( p_node );
-}
-
 bool input_item_HasErrorWhenReading( input_item_t *p_item )
 {
     vlc_mutex_lock( &p_item->lock );
