@@ -2,7 +2,7 @@
  * image.c : wrapper for image reading/writing facilities
  *****************************************************************************
  * Copyright (C) 2004-2007 VLC authors and VideoLAN
- * $Id: 9b4631ecaa442bed307b73991c1ed60c6903db29 $
+ * $Id: 20582ef1d6759a02d7d05b6dbcd45120c72849aa $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -655,7 +655,7 @@ static decoder_t *CreateDecoder( vlc_object_t *p_this, video_format_t *fmt )
     p_dec->pf_vout_buffer_new = video_new_buffer;
 
     /* Find a suitable decoder module */
-    p_dec->p_module = module_need( p_dec, "decoder", "$codec", false );
+    p_dec->p_module = module_need( p_dec, "video decoder", "$codec", false );
     if( !p_dec->p_module )
     {
         msg_Err( p_dec, "no suitable decoder module for fourcc `%4.4s'. "
