@@ -2,7 +2,7 @@
  * playlist_internal.h : Playlist internals
  *****************************************************************************
  * Copyright (C) 1999-2008 VLC authors and VideoLAN
- * $Id: 78b4b5c46d659dfc72dfc017baafe418cb399ea3 $
+ * $Id: ed40679f8f9f77936c5cd5c8816255ca6dfa4aca $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Clément Stenac <zorglub@videolan.org>
@@ -89,7 +89,7 @@ typedef struct playlist_private_t
     bool     b_preparse; /**< Preparse items */
 } playlist_private_t;
 
-#define pl_priv( pl ) ((playlist_private_t *)(pl))
+#define pl_priv( pl ) container_of(pl, playlist_private_t, public_data)
 
 /*****************************************************************************
  * Prototypes
